///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST II
// Ying Feng
// Computer Science, Indiana University
// Feb-Aug 2003
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


package absurdist;

import java.text.*;
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST II
// Ying Feng, Vladimir Menkov
// Computer Science, Indiana University
// Feb-Oct 2003
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
import java.util.*;
import java.io.Serializable;
 
import driver.*;


/** This is a special subclass of Concept, in which each instance
    contains an internal feature vector used to compute external similarity.
*/
public class FeatureVectorConcept extends Concept {
    /** An index-value pair; used as a component of fv. 
	@see #fv
     */
    public static class IVPair {
	int idx;
	double val;
	public IVPair( int idx,	double val) {
	    this.idx=idx;
	    this.val=val;
	}
    };

    /** The fv vector contained in each  FeatureVectorConcept is
	used to determine "distance" between concepts, instead
	of the (d1,d2) pair in the parent Concept class. Such a vector
	can be used to describe, for example, the frequency of
	a term's occurrences in the paragraphs of a document,
	or in the documents of a corpus.
    */
    public Vector fv;

    public FeatureVectorConcept( String name, String category, int rank, 
				 Vector fv) {
	super(name, category, rank,0,0);
	this.fv = fv;	
	// FIXME: need to make sure the vector is sorted
    }

  /** Computes the normalized semantic distance between this concept
     and the specified one. The distance is computed as
     <pre>
     distance(a,b) = 1 - similarity(a,b),
     </pre>
     where similarity(a,b) is a Tanimoto coefficient
     f(a&b)/f(a|b)
  */
    public double distance( Concept concept )    {
	if (concept instanceof FeatureVectorConcept) {
	    FeatureVectorConcept x = ( FeatureVectorConcept)concept;
	    int i1=0, i2=0;
	    int common=0;
	    while(i1<fv.size() && i2<x.fv.size()) {
		int p1 = ((IVPair)fv.elementAt(i1)).idx;
		int p2 = ((IVPair)x.fv.elementAt(i2)).idx;
		if (p1 < p2) {
		    i1 ++;
		} else if (p1 > p2) {
		    i2 ++;
		} else if (p1==p2){
		    if ( ((IVPair)fv.elementAt(i1)).val!=0 &&
			 ((IVPair)x.fv.elementAt(i2)).val !=0) {
			common ++;
		    }
		    i1 ++;
		    i2 ++;
		}
	    }
	    double sim = (double)common/(double)(fv.size()+x.fv.size()-common);
	    return 1.0 - sim;
	} else {
	    // A concept from a completely different space; no similarity
	    return 1.0;
	}

    }

}
