///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST II
// Ying Feng, Vladimir Menkov
// Computer Science, Indiana University
// Feb-Aug 2003
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


package absurdist;
    

/** An auxiliary class used to form keys in the link similarity cache
    in Absurdist. It is hoped that such a key is more efficient than
    a string. Maybe not :-)
*/
class LinkPair {
    int io, jo;
    int in, jn;

    LinkPair(Link lo, Link ln) {
	io = lo.i;
	jo = lo.j;
	in = ln.i;
	jn = ln.j;	    
    }

    /** A convenience constructor */
    LinkPair(int io, int jo, Link ln) {
	this.io = io;
	this.jo = jo;
	this.in = ln.i;
	this.jn = ln.j;	    
    }

    /** A convenience constructor */
    LinkPair(Link lo, int in, int jn) {
	this.io = lo.i;
	this.jo = lo.j;
	this.in = in;
	this.jn = jn;
    }

    /** Returns a string representation of this link pair. */
    public String toString() {
	return "("+io+","+jo+"; "+in+","+jn+")";
    } 
    
    /** Used in hashing */
    //static final int q = 31*31;

    /** Just something... I am afraid the standard hashCode() does
	not work right */
    public int hashCode() {
	/*
	int c = io;
	c = c*q + in;
	c = c*q + jo;
	c = c*q + jn;
	*/
	int c = (((((io << 8) + jo ) << 8) + in)<<8 ) + jn;
	return c;
    }

    /** Returns true if the specified linkpair is the same as this one. 
     (Looks like you got to have your own equals() if you use this class
    as a Hashtable key.) */
    public boolean equals(Object _o) {
	if (_o instanceof LinkPair) {
	    LinkPair o = (LinkPair)_o;
	    return o.io==io && o.jo==jo && o.in==in && o.jn==jn;
	} else return false;
    }
}

