///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST II
// Ying Feng, Vladimir Menkov
// Computer Science, Indiana University
// Feb-Oct 2003
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

package absurdist;
  
/** The interface for a structure used to cache information about 
    link similarity. Used in ABSURDIST's for fast excitation computation.
    The interface can be implemented as a hash table using the 
    (Link,Link) pair as a key, or in a more complex way.
*/

interface LinkSimCache {

    /** Tell the table how much data will be stored: maximum
	number of distinct lo and ln Links. The concrete class
	implementing LinkSimCache may do something with these
	data (e.g., allocate arrays), or just ignore them.
	@param loCnt How many distinct values for the first argument
	to put() are expected.
	@param lnCnt How many distinct values for the second argument
	to put() are expected.
    */
    void setSizes(int loCnt, int lnCnt);

    /** Stores the value representing Sim(lo,ln). Returns
	the value previously stored in the table (NaN if
	none was stored)
    */
    double put( Link lo,  Link ln, double val);

    /** Retrieves the value representing Sim(lo,ln), or
	NaN if	none was stored)
    */
    double get( Link lo,  Link ln);

    /** Returns the number of values stored */
    int size();

}
