///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST II
// Ying Feng, Vladimir Menkov
// Computer Science, Indiana University
// Feb-Oct 2003
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


package absurdist;
  
import java.util.*;

/** LinkSimCache implemented as a rectangular array. Only rows and
    columns representing actual non-empty links, plus the EMPTY
    constant, are stored; this is designed to save on the overhead,
    as compared to LinkSimCacheHash. Access may not be as fast as
    with LinkSimCacheHash, though.
    @see LinkSimCache
    @see LinkSimCacheHash

 */

class  LinkSimCacheArray implements  LinkSimCache {

    /** These tables are used to find rows and columns at
	which data for the given links are stored. Each
    table maps a Link to an Integer */
    private Hashtable lo2id=new Hashtable(), ln2id=new Hashtable();

    private double [][] data=null;
    private int loCnt=0, lnCnt=0;

    /** Allocates the arrays based on the sizes specified.*/
    public void setSizes(int loCnt, int lnCnt) {
	this.loCnt=loCnt;
	this.lnCnt=lnCnt;
	if (data != null) {
	    throw new IllegalArgumentException("LinkSimCacheArray.setSizes() cannot be called more than once");
	}
	data = new double[loCnt][];
	for(int i=0; i<loCnt; i++) {
	    data[i] = new double[lnCnt];
	}
	//FIXME: it would be nice to fill the arrays with NaNs now, to
	// detect attempted access to uninitialized values
    }
	
    /** Looks up the row or column index for the specified link.
	FIXME: this method should be synchronized, but we don't
	bother, for efficiency's sake
     */
    private final int getIndex( Hashtable l2id, int maxSize, Link link) {
	Integer q = (Integer)l2id.get(link);
	if (q != null) {
	    return q.intValue();
	} else {
	    // Pick and store a new ID.
	    int id = l2id.size();
	    if (id >= maxSize) {
		for(Enumeration e=l2id.keys(); e.hasMoreElements(); ) {
		    Link z = (Link)e.nextElement();
		    System.out.println(z + 
				       (z.equals(link)?" EQ": "") +
				       (z.hashCode()==link.hashCode()?" EH": ""));
		}
		throw new IllegalArgumentException("Attempting to save more links than initially requested (" + id+"); offending link="+link);
	    }
	    l2id.put(link, new Integer(id));
	    return id;
	}
    }

    /** Stores the value representing Sim(lo,ln). Returns
	the value previously stored in the table (NaN if
	none was stored)
    */
    public double put( Link lo, Link ln, double val) {
	int i1=getIndex(lo2id, loCnt, lo);
	int i2=getIndex(ln2id, lnCnt, ln);
	double d = data[i1][i2];
	data[i1][i2] = val;
	return d;
    }

    /** Retrieves the value representing Sim(lo,ln), or
	NaN if	none was stored)
    */
    public double get( Link lo, Link ln) {
	int i1=getIndex(lo2id, loCnt, lo);
	int i2=getIndex(ln2id, lnCnt, ln);
	return data[i1][i2];
    }

    public int size() {
	return loCnt * lnCnt;
    }

}
