///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST II
// Ying Feng
// Computer Scicence, Indiana University
// Feb-Aug 2003
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


package absurdist;

import java.util.*;
import java.io.Serializable;

import driver.*;


/** Class RelationModel
    is the model for a particular type of relation, including specifications on its inherent properties.
*/

public class RelationModel implements Serializable
{
    /** Identifier of this RelationModel.
     */
    public String label = "";
    /** Whether the relation is directed. 
     */
    public boolean directed = false;

    /* for future use
    //  binary, 3-nary, ...,  n-ary 
    public int arity;
    //  importance of the relation when deciding mapping 
    public double weight;
    */

    /** Constructor for a default relation model. 
     */
    public RelationModel()
    {
	label = "Default";
	directed = false;
    }    

    /** Constructs a relation model of the  specified label and direction feature.
     */
    public RelationModel( String label, boolean directed )
    {
	this.label = label;
	this.directed = directed;
    }

    /** Compares the ordering between this model and the specified one, according to their label.
     */
    public int compareTo( RelationModel model )
    {
	return( label.compareTo( model.label ) );
    }

    /** Returns a string representation of this relation model
     */
    public String toString()
    {
	String str = "RelationModel " + label + ":\t";
	if ( directed )
	    str += "Directed";
	else
	    str += "Undirected";
	return str;
    }
}
