///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST II
// Ying Feng, Vladimir Menkov
// Computer Science, Indiana University
// June 2004
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


package bnb;
    
import java.io.*;
import java.text.*;
import java.util.*;

import common.*;  // Classes used both by ABSURDIST and by other our packages
//import driver.*;
import absurdist.*;



/** Finding an optimal permutation (one miniimizing the "mismatched relation"
    measure, i.e. maximizing energy) by a simple branch-and-bound method.
    It should produce the same result as exhaustive search, but a bit faster.

    misRel used internally is twice the misRel computed by Measure.evalMap.
 */

public class BranchAndBound {

    int bestMap[]=null;
    double bestMisRel;

    int no, nn;
    Absurdist absurdist;

    int map[];
    boolean usedJ[];

    public BranchAndBound(Absurdist absurdist) {
	no = absurdist.correspond[0].length;
        nn = absurdist.correspond[0][0].length;
	if (no != nn) throw new IllegalArgumentException("Sorry, non-square matrices ("+no +" by "+ nn+") not supported\n");
	this.absurdist = absurdist;
	map = new int [no];
	usedJ = new boolean[no];
	tryNext(0, 0);	
    }
    
    public int[] getMap() {
	if (bestMap == null) {
	    throw new AssertionError("BranchAndBound found nothing!?");
	}
	return bestMap;
    }



    /**
       
       @param map In map[0:no-1], positions map[0:inCnt-1] are already filled
     */
    void tryNext(int filledCnt, 
		 double filledMisRel) {
	
	//System.out.print("BB: try {");
	//for(int i=0; i<filledCnt; i++) System.out.print(" " + map[i]);
	//System.out.println("}, misRel=" +filledMisRel );

	if (filledCnt == map.length) {
	    // Are we there yet?
	    if (bestMap == null ||  filledMisRel<bestMisRel) {
		// This permutation is the best so far!
		bestMap = new int[map.length];
		for(int i=0; i<map.length; i++) bestMap[i]=map[i];
		bestMisRel = filledMisRel;
		//System.out.println("BB: New Optimum!");
	    }
	    return;
	}

	int no = map.length;
	
	for(int j=0; j<nn; j++) {
	    if (usedJ[j]) continue;
	    // Try to put  map[filledCnt] := j, and see how
	    // it will increase misRel
	    
	    double deltaMisRel = 0;
	    for(int i=0; i<filledCnt; i++) {
		// Compare (i,filledCnt) and (map[i],j)
		double sim = absurdist.calcLinkSim( new Link( i, filledCnt ),
						    new Link( map[i], j ));
		deltaMisRel += Math.abs(1-sim);
		// Compare (filledCnt,i) and (j,map[i])
		sim = absurdist.calcLinkSim( new Link( filledCnt, i ), 
					     new Link( j, map[i]));
		deltaMisRel += Math.abs(1-sim);
	    }

	    if (absurdist.alpha !=0) {
		double exSimFunc = 0;	    
		for ( int io=0; io<filledCnt;  io++ ) {
		    exSimFunc += absurdist.exSim[io][j];
		}
		for ( int io=0; io<filledCnt;  io++ ) {
		    int jn = map[io];
		    exSimFunc += absurdist.exSim[filledCnt][jn];
		}
		deltaMisRel+=2*(absurdist.alpha/absurdist.beta)*(no-1)*exSimFunc;
	    }

	    if (bestMap!=null && filledMisRel + deltaMisRel >= bestMisRel) {
		// no sense to go this way: there is already a full
		// permutation with the same, or lower, misRel!
		continue;
	    }
	    

	    map[filledCnt] = j;
	    usedJ[j] = true;
	    tryNext(filledCnt+1, filledMisRel + deltaMisRel);
	    usedJ[j]=false;
	}

    }

  

}
