///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST II
// Ying Feng
// Computer Scicence, Indiana University
// Feb-Aug 2003
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



package driver;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.File;

import absurdist.*;
import graph.*;
import gui.*;
import penalty.*;



/** Class ApplicDriver
    is used to run AppletDriver as an application, rather than an applet. 
    This provides a visual interface to the Absurdist code without all the 
    limitations (such as those on file access) that using an applet imposes.
*/
public class ApplicDriver extends JFrame 
{
    static boolean debugMode = true;
    AbsPanel abs = null;
    ////AppletDriver testApplet = null;

    // GUI components - menu items
    private MenuItem openItem;          // file menu
    private MenuItem saveItem;
    //// private MenuItem saveAsItem;
    private MenuItem importSourceItem;
    private MenuItem importTargetItem;
    private MenuItem exportItem;
    private MenuItem exitItem;


    public ApplicDriver() 
    {
	super( "Absurdist GUI" );

	MenuBar menuBar = new MenuBar();	
	setMenuBar( menuBar );
	MenuHandler menuHandler = new MenuHandler();
	
	// file menu
	Menu fileMenu = new Menu( "File" );
	////fileMenu.setMnemonic( 'F' );
	menuBar.add( fileMenu );
	openItem = new MenuItem( "Open Session");//, 'O' );
	openItem.addActionListener( menuHandler );
	fileMenu.add( openItem );
	fileMenu.addSeparator(); // separate line
	saveItem = new MenuItem( "Save Session");//, 'S' );
	saveItem.addActionListener( menuHandler );
	fileMenu.add( saveItem );
	/*
	saveAsItem = new MenuItem( "Save As");//, 'A' );
	saveAsItem.addActionListener( menuHandler );
	fileMenu.add( saveAsItem );
	*/
	fileMenu.addSeparator(); // separate line
	importSourceItem = new MenuItem( "Import Source");//, 'I' );
	importSourceItem.addActionListener( menuHandler );
	fileMenu.add( importSourceItem );
	importTargetItem = new MenuItem( "Import Target");//, 'I' );
	importTargetItem.addActionListener( menuHandler );
	fileMenu.add( importTargetItem );
	exportItem = new MenuItem( "Export Results");//, 'E' );
	exportItem.addActionListener( menuHandler );
	fileMenu.add( exportItem );
	fileMenu.addSeparator(); // separate line
	exitItem = new MenuItem( "Exit");//, 'x' );
	exitItem.addActionListener( menuHandler );
	fileMenu.add( exitItem );

	int width = 1000;
	int height = 900;
	setSize( width, height );	
	setLocation( 100, 0 );	
	
	abs = new AbsPanel( width, height );
	Container c = getContentPane();
	c.setLayout( new BorderLayout() );
	c.add( abs );
	show();
    }


    public static void debug( String s )
    {
        if ( debugMode )
            System.out.print( s );
    }
        
    public static void debugln( String s )
    {
        if ( debugMode )
            System.out.println( s );
    }

    public static void main(String argv[]) 
    {	
	/*
	Absurdist.maxLoop = 
	    Integer.getInteger( "maxLoop", Absurdist.maxLoop).intValue();
	Absurdist.intLoop = 
	    Integer.getInteger( "intLoop", Absurdist.intLoop).intValue();
	debugln("(max iter)      maxLoop = " +  Absurdist.maxLoop);
	debugln("(iter sampling) intLoop = " +  Absurdist.intLoop);
	*/

	ApplicDriver app = new ApplicDriver();
	app.addWindowListener( new WindowAdapter() {
	    public void windowClosing( WindowEvent e )
		{
		    System.exit( 0 );
		}
	    });
    }


    /** Class MenuHandler: handling all menu events. */
    private class MenuHandler implements ActionListener
    {
	public void actionPerformed ( ActionEvent e )
	{	    	    
	    // file menu
	    if ( e.getSource() == openItem )
		abs.openSession(); 
	    else if ( e.getSource() == saveItem ) 
		abs.saveSession(); 	    
	    else if ( e.getSource() == importSourceItem ) 
		abs.importSystem( true ); 	    
	    else if ( e.getSource() == importTargetItem ) 
		abs.importSystem( false ); 	    
	    if ( e.getSource() == exportItem ) 
	        abs.exportResults(); 
	    else if ( e.getSource() == exitItem ) 
		System.exit( 0 );
	}
    }


    /*  

    // Some commands are special; others begin with "help-" or "q-".
    class GotoMenuItem extends MenuItem {
	GotoMenuItem(String label, final String cmd) {
	    super(label);
	    setActionCommand(cmd);
	    addActionListener( new GotoMenuItemListener());
	}
    }

    class GotoMenuItemListener implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    String cmd = e.getActionCommand();
	    try {

		if (cmd.equals("exit")) {
		    System.exit(0);
		} else if (cmd.equals("save")) {
		    if (testApplet != null) {

			Object[] possibilities = null;
			String fileName = (String)JOptionPane.
			    showInputDialog(ApplicDriver.this,
					    "Enter output file name:",
					    "File name?",
					    JOptionPane.PLAIN_MESSAGE,
					    null,
					    possibilities,
					    "tmp.out");

			if ((fileName != null) && (fileName.length() > 0)) {
			    testApplet.absurdist.saveSystems(fileName);
			} else {
			    //If you're here, the return value was null/empty.
			    System.err.println("No file name specified");
			}
		    }
		} else if (cmd.equals("load")) {
		    if (testApplet != null) {

			Object[] possibilities = null;
			String fileName = (String)JOptionPane.
			    showInputDialog(ApplicDriver.this,
					    "Enter input file name:",
					    "File name?",
					    JOptionPane.PLAIN_MESSAGE,
					    null,
					    possibilities,
					    "tmp.out");

			if ((fileName != null) && (fileName.length() > 0)) {
			    testApplet.loadSystems(fileName);
			} else {
			    //If you're here, the return value was null/empty.
			    System.err.println("No file name specified");
			}

		    }
		} else {
		}
	    } catch(Exception ex) {
		System.err.println(ex);
	    }
	}
    }

    GotoMenuItem addItem( Menu m, String label, final String cmd, int key) {
       GotoMenuItem mi = new GotoMenuItem(label, cmd);
       if (key != 0) mi.setShortcut( new MenuShortcut(key));
       m.add(mi);
       return mi;
    }

    */
}


