///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST++
// Ying Feng
// Computer Scicence, Indiana University
// Feb-Aug 2003
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////





package driver;

import java.util.*; 
import java.io.*; 

import absurdist.*;


/** Class SimpleDriver:    
   The main class for a program that runs Absurdist on two specified
   concept systems read from input files
*/
public class CmpDriver
{
    static boolean debugMode = false;

    public static void debug( String s )
    {
        if ( debugMode )
            System.out.print( s );
    }
        
    public static void debugln( String s )
    {
        if ( debugMode )
            System.out.println( s );
    }
        
    public static void error( String s )
    {
        //// if ( errorMode )
        System.err.println( s );
    }
        
    public static void main( String[] args )    {
        if ( args.length != 2 ) {
	    String msg =
		"Usage: java [-DL=0.01] [-DmaxLoop=2000] [-DresetChi=true]  [-DresetL=true] [-Dexp=false] CmpDriver file1 file2\n" +
		"Defaults: resetChi=true, exp=false";
            error( msg);
            return;
        }
        
        try {
            String name1 = args[0];
            String name2 = args[1];
	    System.out.println("Reading system 1 from file: " + name1);
	    ConceptSystem systemO = ConceptSystem.readTxt( name1 );
	    System.out.println("Reading system 2 from file: " + name2);
	    ConceptSystem systemN = ConceptSystem.readTxt( name2 );

	    // Ensure that Absurdist will be created without 
	    // expensive history arrays
	    Absurdist.turnOffHistory();

	    // learning rate (use default if none is supplied)
	    String flag =  "L" ;
	    if (System.getProperty(flag)!=null) {
		Absurdist.lRate =Double.parseDouble(System.getProperty(flag));
	    }
	    System.out.println( "Learning rate: -DL=" +Absurdist.lRate);

	    // step count (use default if none is supplied)
	    flag =  "maxLoop" ;
	    if (System.getProperty(flag)!=null) {
		Absurdist.maxLoop =Integer.parseInt(System.getProperty(flag));
	    }
	    System.out.println( "maxLoop=" +Absurdist.maxLoop);

	
            //// ConceptSystem systemN = new ConceptSystem( systemO, noise );
            Absurdist absurdist = new Absurdist( systemO, systemN );

	    // resetChi: Default is true
	    final String flag2 =  "coefMode" ;
	    System.out.println( "-D"+flag2+"=" +absurdist.coefMode);

	    // exp: Default is false
	    absurdist.useExponential = Boolean.getBoolean( "exp" );
	    System.out.println( "-Dexp=" + absurdist.useExponential);

	    //test(systemO, systemN); System.exit(0);

            absurdist.mapSystem( );
            absurdist.print( );
       }
        catch ( Exception e ) {
            e.printStackTrace();
            System.exit(0);
        }
    }

    /** Print out brief information about a bundle of relations */
    static void showRelationBundle( Vector rso, PrintStream out)    {
	out.print("{");
	if ( rso == null ) { } // don't do anything
	else {
	    for ( int ko=0; ko < rso.size(); ko++ ) {
		Relation ro = ( Relation ) rso.get(ko);
		out.print("(" + ro.model.label + ": " + ro.weight + ") ");
	    }
	}
	out.println("}");
    }

    
    static void test(ConceptSystem systemO , ConceptSystem systemN) {
	int i=0;
	for(int j=0; j<systemO.getConceptNum(); j++) {
	    Vector v1 = systemO.getRelations( i, j);
	    Vector v2 = systemO.getRelations( j, i);
	    System.out.println("Rel(" +i+","+j+"):");
	    showRelationBundle(v1, System.out);
	    System.out.println("Rel(" +j+","+i+"):");
	    showRelationBundle(v2, System.out);
	}
    }

}
