///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST++
// Ying Feng
// Computer Scicence, Indiana University
// Feb-Aug 2003
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////





package driver;

import java.text.*; 

import absurdist.*;


/** Class RandomDriver:
   The main class to run a simple test for ABSURDIST (no GUI).
   Generates a random concept system, another system with noise,
   and matches the two.
*/
public class RandomDriver
{
    static boolean debugMode = false;

    public static void debug( String s )
    {
        if ( debugMode )
            System.out.print( s );
    }
        
    public static void debugln( String s )
    {
        if ( debugMode )
            System.out.println( s );
    }
        
    public static void error( String s )
    {
        //// if ( errorMode )
        System.err.println( s );
    }
        
    public static void main( String[] args )
    {
        if ( args.length < 4 ) {
            error( "Usage: java driver.RandomDriver name number_of_concepts edge_fullness noise [repeat_cnt]" );
            return;
        }
        
        try {
            String name = args[0];
            int n = Integer.parseInt( args[1] );
            double fullness = Double.parseDouble( args[2] );
            double noise = Double.parseDouble( args[3] );
	    int repeatCnt = 1;
	    try {
		repeatCnt = Integer.parseInt( args[4] );
	    } catch(Exception e) {}

	    Absurdist.Results avg=new Absurdist.Results();

	    for(int i=0; i<repeatCnt; i++) {

            ConceptSystem systemO = new ConceptSystem( n, fullness );
            ConceptSystem systemN = new ConceptSystem( systemO, 0, 0, noise, 0 );
            //// ConceptSystem systemN = new ConceptSystem( systemO, noise );
	    Absurdist.turnOffHistory();
            Absurdist absurdist = new Absurdist( systemO, systemN );
	    boolean expMode = true;
            absurdist.mapSystem(  );
            absurdist.print( );
            /*
            absurdist.printSystem();
            absurdist.printCorrespond();
            absurdist.printMapping();
            */
	    

	    avg.add(absurdist.results);

	    }


	    // Get the averages and standard deviations
	    avg.averages();

	    DecimalFormat df = new DecimalFormat( "0.000" );

	    System.out.println("======= Average values for " + avg.rpt +
			       " experiments ========");
	
	    System.out.println("Iteration count                       : " +
			       df.format(avg.iterCnt)+" +- "+
			       df.format(avg.iterCntDev));
	    System.out.println("Total perfect mappings                : " + 
				avg.perfectCnt + 
				", percentage: " + 
				df.format(((double)avg.perfectCnt)/avg.rpt) );
	    System.out.println("Mismapped concepts (identity mpg only): " +
			       df.format(avg.misCon) +" +- "+
			       df.format(avg.misConDev));
	    System.out.println("Mismapped relations                   : " +
			       df.format(avg.misRel )+" +- "+
			       df.format(avg.misRelDev));
	    System.out.println("Adjusted-id Mismapped relations       : " +
			       df.format(avg.misRelAdjId )+" +- "+
			       df.format(avg.misRelAdjIdDev));
	    System.out.println("Adjusted-op Mismapped relations       : " +
			       df.format(avg.misRelAdjOp )+" +- "+
			       df.format(avg.misRelAdjOpDev));
	    System.out.println("'Accuracy'                            : " +
			       df.format(avg.accuracy)+" +- "+
			       df.format(avg.accuracyDev));

        }
        catch ( Exception e ) {
            e.printStackTrace();
            System.exit(0);
        }
    }
}
