package graph;
import java.io.*;
import java.lang.*;

/** The main program for a test driver.
 */
public class GenGraphs {

    /** Tries to generate a random asymmetric graphs.

	Note: this method  may sit in an almost-infinite loop (and then 
	throw an Exception after 
	after 1000 tries) if there are no or very few asymmetric graphs
	of the desirable size. In practice, this never happens with N>=10
	and a high enough D. (D>=5 is fine for N = 10... 200; 
	D>=10 is fine for N=100 .. 10000)

	@param N number of nodes in the graph
	@param D the average degree of a node in the graph
    */
    public static Graph genAsymGraph(int N, double D) throws Exception {
	
	final int maxTry=1000;
	for(int tryCnt=0; tryCnt<maxTry; tryCnt++) {
	    Graph g = new Graph(N, D);
	    int nTryInv = 10;  // try no more than first 10 invariants
	    if (N < nTryInv) nTryInv=N;
	    boolean sym = g.mayBeSymmetric(nTryInv);
	    if (sym) {
		// System.out.println("May be symmetric");	    
		// System.out.print("[SKIP] ");  
		continue;
	    } else {
		return g;
	    }
	}

	String msg = 
	    "\nTried " + maxTry + " graphs and have not found a single asymmetric one.\n Maybe there simply are not any asymmetric graphs with " + N + " vertices and\n" + D + " average degree? Or may be they are quite rare.";				   
	System.out.println(msg);
	throw new Exception("Cannot generate an asymmetric graph");
    }


   /** Generates a specified number of random asymmetric graphs.
	There is no guarantee, of course, that all generated graphs
	will be different, or non-isomorphic.

	Note: this program may sit in an almost-infinite loop (and then stop
	after 1000 tries) if there are no or very few asymmetric graphs
	of the desirable size. In practice, this never happens with N>=10
	and a high enough D. (D>=5 is fine for N = 10... 200; 
	D>=10 is fine for N=100 .. 10000)
    */
    public static void main(String[] arg) throws Exception {
	Graph.debug=false;
	NodeInfo.debug=false;
	if (arg.length != 3) {
	    System.out.println("Usage: Graph N D graph-count\n");
	    System.exit(1);
	}
	int N = Integer.parseInt(arg[0]);
	double D = Double.valueOf(arg[1]).doubleValue();
	int cnt = Integer.parseInt(arg[2]);
	System.out.println("Producing " + cnt + " asymmetric random graphs with " + N +
			   " vertices, avg degree = " + D);
	int doneCnt=0;
	for(; doneCnt<cnt; doneCnt++) {
	    Graph g = genAsymGraph(N, D);
	    System.out.println("\nAsymmetric graph No. " + doneCnt+":");
	    System.out.println(g);
	}
	//System.out.println("Tried " + tryCnt + " random graphs, produced " + doneCnt + " asymmetric ones\n");
    }

}
