package graph;
/** Is used for sorting nodes with respect to topological
    invariants */
public class NodeInfo implements Comparable {
    static boolean debug=true;
    
    /** Node id; Graph.node[id] is the appropriate entry in the
	graph */
    int id;
    /** Sorting key; is made from composing all  topological
	invariants that we have tried so far */
    String key="";
    
    NodeInfo(int i) {
	id = i;
    }
    
    /** Used to sort entries by key */
    public int compareTo(Object o) {
	if (o instanceof NodeInfo) {
                NodeInfo other=(NodeInfo)o;
		return key.compareTo(other.key);
	} else {
	    throw new IllegalArgumentException("not Comparable: " + o);
	}
    }


    /** Do all elements in the sorted array have the same comparison
	key? */
    static boolean allKeysDifferent(NodeInfo w[]) {
	for(int i=0; i<w.length-1; i++) {
	    if (w[i+1].key.equals( w[i].key)) return false;
	}
	return true;
    }

    /** Converts an array of integers into a single "fingerprint"
	integer, using a hash function */
    static int hashArray(Integer r[]) {
	StringBuffer s=new StringBuffer();
	int lastVal=0;
	int cnt=0;
	for(int i=0; i<r.length; i++) {
	    int z = r[i].intValue();
	    if (cnt==0) {
		lastVal = z;
		cnt=1;
	    } else if (z == lastVal) {
		// the last value repeats
		cnt ++;
	    } else {
		// different value
		if (cnt > 1) {
		    s.append(Integer.toHexString(cnt) + "*");
		}
		s.append(Integer.toHexString(lastVal) + ":");
		lastVal = z;
		cnt=1;		
	    }
	}
	if (cnt > 0) {
	    // take care of the last value
	    if (cnt > 1) {
		s.append(Integer.toHexString(cnt) + "*");
	    }
	    s.append(Integer.toHexString(lastVal) + ":");
	}
	return s.toString().hashCode();
    }

}
    
