///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST++
// Ying Feng
// Computer Scicence,Indiana University
// Feb-Aug 2003
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



package gui; 

import java.util.*;
import java.awt.*;
import java.io.Serializable;

//// import java.awt.geom.*;

import absurdist.*;
import driver.*;



/** 
    Class ConceptGraph:
    Visual representation of a concept system as a graph.
*/
public class ConceptGraph extends ConceptSystem implements Serializable
{
    /** Rectangle bound:  defines the bounding borders of the graphical representation 
     */
    /** top-left corner of its bound */
    int x = 0, y = 0;     
    /** width and height of its bound */
    int width = 0, height = 0;

    /** Creates a concept graph from a concept system.
     */
    public ConceptGraph( ConceptSystem system, int x, int y, int width, int height )
    {
	this.x = x;
	this.y = y;
	this.width = width;
	this.height = height;

	name = system.name;
	categories = system.categories;
	models = system.models;

        int i;
        int n = system.getConceptNum();
        for ( i=0; i<n; i++ )
            concepts.add( new ConceptNode( system.getConcept(i), system, this ) );               
    
        Enumeration enum = system.relations.keys();        
        while ( enum.hasMoreElements() ) {
	    String key = ( String )enum.nextElement();
	    Vector rellist = ( Vector )system.relations.get( key );
	    Vector linklist= new Vector();
	    for ( i=0; i < rellist.size(); i++ ) {		
		Relation relation = ( Relation )rellist.get(i);
		RelationLink link = new RelationLink( relation, system, this );   
		linklist.add( link );
	    }
	    relations.put( key, linklist );                    
        }

	/*
        for ( i=0; i<n; i++ ) 
            for ( j=i+1; j<n; j++ ) {        
                if ( system.nRelType <= 1 ) {
                    Relation relation = system.getRelation( i, j, true );
                    if ( relation != null ) { 
                        RelationLink link = new RelationLink( relation, system, this );   
                        relations.put( ConceptSystem.getKey( i, j, true ), link );                    
                    }           
                    relation = system.getRelation( j, i, true );
                    if ( relation != null ) { 
                        RelationLink link = new RelationLink( relation, system, this );   
                        relations.put( ConceptSystem.getKey( j, i, true ), link );                    
                    }           
                }
                else {
		    //// Test.debugln( "System " + system.name + " has " + system.getRelationNum() + " relations" );
                    for ( int k=0; k < system.nRelType; k++ ) {      
			String label = "R"+k;
                        Relation relation = system.getRelation( i, j, label );
                        if ( relation != null ) { 	                    
                            RelationLink link = new RelationLink( relation, system, this );   
                            relations.put( ConceptSystem.getKey( i, j, label ), link );  
			    //// Test.debugln( "Put a link: " + link );
                        }
			if ( Relation.isDirected( label ) ) {
			    relation = system.getRelation( j, i, label );
			    if ( relation != null ) { 	                    
				RelationLink link = new RelationLink( relation, system, this );   
				relations.put( ConceptSystem.getKey( j, i, label ), link );  
				//// Test.debugln( "Put a link: " + link );
			    }
                        }
                    }
                }                    
            }           
	*/
        //// bound = new Rectangle( x, y, width, height );
    }
    
    /** Gets the ith concept node.
     */
    public ConceptNode getNode( int i )
    {
        return ( ConceptNode )getConcept(i);
    }

    /** Gets the relation link list between concept node i and concept node j.
     */
    public Vector getLinks( int i, int j )
    {
        return getRelations( i, j );
    }

    /** Gets the relation link of the specified label between concept node i and concept node j.
     */
    public RelationLink getLink( int i, int j, String label )
    {
	return ( RelationLink )getRelation( i, j, label );
    }    
    
    /** Paints the object.
     */
    public void paint( Graphics2D g )
    {
	// frame
        g.setColor( Color.lightGray );
	g.setStroke( new BasicStroke(5) );
	g.drawRect( x, y, width, height );
        //// g.drawRect( bound.getX(), bound.getY(), bound.getWidth(), bound.getHeight() );

	// nodes
        for ( int i=0; i < concepts.size(); i++ ) 
            getNode(i).paint(g); 

	// links
        Enumeration enum = relations.elements();
        while ( enum.hasMoreElements() ) {            
            Vector linklist = ( Vector )enum.nextElement();
	    for ( int i=0; i < linklist.size(); i++ ) {		
		RelationLink link =  ( RelationLink )linklist.get(i);
		link.paint(g);
	    }
        }                      
    }        
    
    public String toString()
    {
        String str = "ConceptGraph " + name + " ----------\n";
	int i;

        str += "\nConcept categories ----- \n{ ";
        for ( i=0; i < getCategoryNum(); i++ ) {
            str += getCategory(i);
	    if ( i < getCategoryNum()-1 )
		str += ", ";
	    else
		str += " }\n";
	}

        str += "\nRelation Models ----- \n";
        for ( i=0; i < getModelNum(); i++ ) 
            str += getModel(i) + "\n";

        str += "\nConcept Nodes ----- \n"; 
	str += "Concept Name\tCategory\tRank\tValues\n"; 
	str += " ==> Node Coordinates\tColor\tSize\n"; 
        for ( i=0; i < getConceptNum(); i++ ) 
            str += getNode(i) + "\n";

        str += "\nRelation Links ----- \n";
	str += "Relation Label\tConcepts\tWeight\n";
	str += " ==> Link: \tColor\tWidth\n";		
        Enumeration enum = relations.elements();        
        while ( enum.hasMoreElements() ) {
            Vector linklist = ( Vector )enum.nextElement();
	    for ( i=0; i < linklist.size(); i++ )
		str += ( RelationLink )linklist.get(i) + "\n";
        }

        return str;    
    }
}
