///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST++
// Ying Feng
// Computer Scicence, Indiana University
// Feb-Aug 2003
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



package gui; 

import java.awt.*;
import java.util.*;

import absurdist.*;
import driver.*;



/** 
    Class ConceptNode:
    Visual representation of a concept in a concept system graph.
*/
public class ConceptNode extends Concept
{
    /**olor set for mapping concept categories */
    public final static Color[] colors = { Color.red, Color.green, Color.blue,
					   Color.yellow.darker(), Color.cyan.darker(), Color.magenta.darker() };
    public final static int D = 16; // default node diameter
     
    Color color = Color.black; 	// color in predefined color set
    int alpha = 0;              // transparency
    int size = 0;              	// size in pixel
    int x = 0, y = 0;		// center position in pixel
    //// Concept concept = null;	// corresponding concept
    
    /** Creates a concept node by encoding its concept features. */
    public ConceptNode( Concept concept, ConceptSystem system, ConceptGraph graph )
    {
        super( concept );
        name = concept.name;
        color = cat2color( category, graph.categories );
	alpha = rank2alpha( rank );
        //// size = rank2size( rank );
	size = D;
	if ( system.isValued() ) { // map 2D feature values to 2D positions
	    x = dim2pos( d1, graph.x, graph.width );
	    y = dim2pos( d2, graph.y, graph.height );
	}
	else { // no feature value, use default, i.e. regular polygon layout
	    int index = system.getConceptIndex( concept );
	    double theta = index * 2*Math.PI / system.getConceptNum();
	    int xo = graph.x + graph.width/2;
	    int yo = graph.y + graph.height/2;
	    int a = (int)( 0.4 * graph.width );
	    int b = (int)( 0.4 * graph.height );
	    x = dim2pos( Math.cos( theta ), xo, a );
	    y = dim2pos( Math.sin( -theta ), yo, b );
	}
    }

    /*

    // Create a concept node by encoding its concept features.
    public ConceptNode( Concept concept, Rectangle bound )
    {
        super( concept );
        this.color = type2color( type );
        this.size = rank2size( rank );
        this.x = dim2PosX( d1, bound );
        this.y = dim2PosY( d2, bound );
    }

    public Node( Concept concept, Color color, int size, int x, int y )
    {
        this.concept = concept;
        this.color = color;
        this. size = size;
        this.x = x;
        this.y = y;
    }

    // Decode this visual node into corresponding semantic concept.
    public decode()
    {
    }

    */

    /** Encodes concept type into node color. */
    public static Color cat2color( String category, Vector categories ) 
    {
	Color color = Color.black;
	if ( category.equals( "Default" ) )
	    color = colors[0];
        else {
            //// int catIdx = Integer.parseInt( category.substring(4) );
	    int catIdx = categories.indexOf( category );
            color = colors[ catIdx % colors.length ];
        }
	return color;
    }

    /** Encodes concept rank into node transparent. */
    public static int rank2alpha( int rank )
    {
	int alpha = 255 * rank / RANK;	
	return alpha;
    }

    /** Encodes concept rank into node size. *
    public static int rank2size( int rank )
    {
	int size = D + rank * D / RANK;	
	return size;
    }
    */

    /** Encodes concept dimension value into node position coordinates.
     */
    public static int dim2pos( double d, int start, int span )
    {
	int pos = start + ( int )( d * span );
        return pos;
    }

    /*

    // Encode concept dimension 1 value into node position x-coordinates.
    public static int dim2PosX( double d1, int x, int width )
    {
	int xPos = x + ( int )( d1 * width );
        return xPos;
    }

    // Encode concept dimension 2 value into node position y-coordinates.
    public static int dim2PosY( double d2, int y, int height )
    {
        int yPos = y + ( int )( d2 * height );
	return yPos;
    }
  
    // Get x position of the center point.
    public int getCenterX()
    {
        return ( x + size/2 );
    }
    
    // Get y position of the center point.
    public int getCenterY()
    {
        return ( y + size/2 );
    }

    // Get the center point.
    public Point getCenter()
    {
        return new Point( x+size/2, y+size/2 );
    }

    */
   
    /** Gets the x position of left-top corner. */
    public int getLeft()
    {
        return ( x - size/2 );
    }
    
    /** Gets the y position of left-top corner. */
    public int getTop()
    {
        return ( y - size/2 );
    }

    /** Gets the left-top point. */
    public Point getLeftTop()
    {
        return new Point( x-size/2, y-size/2 );
    }

    /** Computes the geometric distance between this node and the specified one.
     */
    public double distance( ConceptNode node )
    {
	int dx = x - node.x;
	int dy = y - node.y;
	double dis = Math.sqrt( ( double )( dx * dx + dy * dy ) );
	return dis;
    }

    /** Paints the object.
     */
    public void paint( Graphics2D g )
    {
	Color c = new Color( color.getRed(), color.getGreen(), color.getBlue(), alpha );
        g.setColor( c );
	g.fillOval( getLeft(), getTop(), size, size );
	g.setColor( Color.black );
	g.setFont( g.getFont().deriveFont(D) );
	g.drawString( name, x-D/2, y+D/3 );
	//// g.drawString( name, getCenterX()-D/2, getCenterY()+D/3 );
    }
    
    /** String representation.
     */
    public String toString()
    {
	String str = super.toString() + "\n";
	str += " ==> Node ( " + x + ", " + y + " )\t" + color + "\t" + alpha;
	return str;
    }
}
