///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST++
// Ying Feng
// Computer Scicence, Indiana Universit
// Feb-Aug 2003
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


 
package gui;

import java.awt.*;

import absurdist.*;
import driver.*;
import java.text.*;



/** 
    Class MatrixMap
    Visual representation of an Absurdist matrix using cell array map, i.e.
    each element in the matrix will be color coded in its corresponding cell.
    This class assumes the matrix values is between [-1, 1]; 
    values beyond that range will be mapped to as the maximum value.
*/
public class MatrixMap
{
    /** number of pixels each element cell occupies */
    public static final int SC = 2;

    double[][][] matrix;         // matrix the plot is for
    int nStep = 0;               // the actual # of steps recorded in matrix
    
    int x = 0, y = 0;            // top-left corner of its bound
    int width = 0, height = 0;   // width and height of its bound
    

    /** Creates the map for the specified matrix and current step, 
	with the specified position and dimension. */
    public MatrixMap( double[][][] matrix, int nStep, int x, int y, int width, int height )
    {
	this.matrix = matrix;
	this.nStep = nStep;
	this.x = x;
	this.y = y;
	this.width = width;
	this.height = height;	
    }


    /** Sets the matrix. */
    public void setMatrix( double[][][] matrix )
    {
        this.matrix = matrix;
    }
    
    /** Sets the number of steps. */
    public void setStep( int nStep )
    {
        this.nStep = nStep;
    }

    /** Sets the boundary of the map. */
    public void setBound( int x, int y, int width, int height )
    {
        this.x = x;
	this.y = y;
	this.width = width;
	this.height = height;		
    }


    /** Paints the matrix map. */
    public void paint( Graphics2D g )
    {
	// draw frame
        g.setColor( Color.lightGray );
	g.setStroke( new BasicStroke(5) );
	g.drawRect( x, y, width, height );

	// draw cells in map
	int no = matrix[0].length;
	int nn = matrix[0][0].length;
	int cw = width / no; // cell width
	int ch = height / nn; // cell height

	for ( int io=0; io<no; io++ ) {
	    for ( int in=0; in<nn; in++ ) {		
		float alpha = (float)matrix[nStep-1][io][in];
		if ( alpha < -1 )
		    alpha = -1;
		if ( alpha > 1 )
		    alpha = 1;
		Color color = alpha > 0 ?
		    new Color( 0, 1f, 0, alpha ) :
		    new Color( 1f, 0, 0, -alpha );
		g.setColor( color );
		int cx = x + io * cw;
		int cy = y + in * ch;
		g.fillOval( cx, cy, cw, ch );
	    }
	}
    }
}
