///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST++
// Ying Feng
// Computer Scicence, Indiana University
// Feb-Aug 2003
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



package gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import absurdist.*;
import driver.*;



/** 
    Class MatrixPanel:
    The panel on which the maps/plots for correspondence and net input matrices are painted.
*/
public class MatrixPanel extends JPanel
{
    Absurdist absurdist; // corresponding Absurdist reference
 
    MatrixMap netMap;    // map for network input matrix
    MatrixMap corMap;    // map for correspondence matrix
    MatrixPlot netPlot;  // plot for network input matrix
    MatrixPlot corPlot;  // plot for correspondence matrix

    boolean valid = false; // whether the matrices maps/plots are in valid status

    public MatrixPanel() 
    {
    }
     
    public MatrixPanel( Absurdist absurdist )
    {
        this.absurdist = absurdist;
    }


    /*
    public void setAbsurdist( Absurdist absurdist )
    {
        this.absurdist = absurdist;        
    }
    */

    /** Initializes the maps/plots for all the matrices, and set the status to valid.
	This method should be called each time Absurdist executes mapping. */
    public void setMatrices()
    {
	// only need to reset them if they are not valid
	if ( !valid ) {
	    netMap = new MatrixMap( absurdist.netIn, absurdist.nStep, 
				    0, 0, getWidth()/4, getHeight() );
	    netPlot = new MatrixPlot( absurdist.netIn, absurdist.nStep, 
				      getWidth()/4, 0, getWidth()/4, getHeight() );
	    corPlot = new MatrixPlot( absurdist.correspond, absurdist.nStep, 
				      getWidth()/2, 0, getWidth()/4, getHeight() );
	    corMap = new MatrixMap( absurdist.correspond, absurdist.nStep, 
				    getWidth()*3/4, 0, getWidth()/4, getHeight() );
	    valid = true;
	}
    }

    /** Sets the current mapping step for all the maps/plots. */
    public void setStep( int nStep )
    {
        netMap.setStep( nStep );
        netPlot.setStep( nStep );
        corPlot.setStep( nStep );
        corMap.setStep( nStep );
    }

    /** 
	Returns true if the matrices map/plots status is valid.
	At any time, the valid/invalid cycle is as follows:
	Absurdist unmapped => matrices invalid
	Absurdist mapping/mapped => matrices valid
    */
    public boolean isValid()
    {
	return valid;
    }

    /** Invalidate all the matrices maps/plots, and clear the display. */
    public void invalidate()
    {
	if ( valid ) {
	    valid = false;
	    repaint();
	}
    }

    /** Paints the component. */
    public void paintComponent( Graphics g )
    {	
        Graphics2D g2D = ( Graphics2D )g;
	super.paintComponent( g2D );       

	// paint the maps/plots for N and C matrix if they are valid
	if ( valid ) { 
	    netMap.paint( g2D );	
	    netPlot.paint( g2D );	
	    corPlot.paint( g2D );	
	    corMap.paint( g2D );	
	}
    }        
}


