///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST++
// Ying Feng
// Computer Scicence, Indiana University
// Feb-Aug 2003
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


 
package gui;

import java.awt.*;
import java.util.*;

import absurdist.*;
import driver.*;



/** Class RelationLink
    Visual representation of a relation in a concept system graph.
*/
public class RelationLink extends Relation
{
    /** color set for mapping relation label types */
    public static Color[] colors = { Color.yellow.darker(), Color.cyan.darker(), Color.magenta.darker(), 
				     Color.red, Color.green, Color.blue };

    /** line styles to indicate relation direction */
    public static final int PLAIN = 0;
    public static final int ARROW = 1;

    /** default link width */
    public static final int W = 2;	

    Color color = Color.black; 	        // color in predefined color set
    int alpha = 0;                      // transparency
    int width = 0;		        // width of the line in pixel
    int style = PLAIN;                  // plain solid line

    /*
       int style = BasicStroke.CAP_BUTT;	// cap style of the line 
       Point ps, pe;		        // starting and ending point
       Relation relation = null;	// corresponding relation
    */

    /*
    public Link( Relation relation, Color color, int width, int style )
    {
        this.relation = relation;
        this.color = color;
        this. width = width;
        this.style = style;
        ps = relation.concepts.get(0).getCenter();
        pe = relation.concepts.get(1).getCenter();
    }
    */
    
    /**Creates a relation link by encoding its relation features.
     */
    public RelationLink( Relation relation, ConceptSystem system, ConceptGraph graph )
    {
	/*
	model = relation.model;
	weight = relation.weight;
	for ( int i=0; i < relation.getArity(); i++ ) {
	    Concept concept = relation.getConcept(i);
	    int index = system.getConceptIndex( concept );
	    ConceptNode node = graph.getNode( index );
	    concepts.add( node );
	}
	*/

	super( relation, 
	       graph.getNode( system.getConceptIndex( relation.getConcept1() ) ), 
	       graph.getNode( system.getConceptIndex( relation.getConcept2() ) ) );

        color = label2color( model, graph.models );
 	alpha = weight2alpha( weight );
	////width = weight2width( weight );
	width = W;
	style = direct2style( isDirected() );

	/*
        if ( relation.isDirected() )
            style = BasicStroke.CAP_SQUARE;
        else
            style = BasicStroke.CAP_BUTT;
        ps = relation.concepts.get(0).getCenter();
        pe = relation.concepts.get(1).getCenter();
	*/
    }

    /*
    // Decode this visual node into corresponding semantic concept.
    public decode()
    {
    }
    */
    
    /**  Encodes relation label into link color. */
    public static Color label2color( RelationModel model, Vector models ) 
    {
	Color color = Color.black;
	if ( model.label.equals( "Default" ) || 
	     model.label.equals( "Directed" ) || model.label.equals( "Undirected" ) )
	    color = colors[0];
        else {
            //// int labelIdx = Integer.parseInt( label.substring(1) );
	    int labIdx = models.indexOf( model );
            color = colors[ labIdx % colors.length ];
        }
        return color;
    }

    /** Encodes relation weight into link transparecncy.*/
    public static int weight2alpha( double weight )
    {
 	int alpha = (int)( 255 * weight );	
	return alpha;
    }

    /** Encodes relation weight into link width.*
    public static int weight2width( double weight )
    {
        int width = ( int )( weight * W );
        return width;
    }
    */

    /** Encodes relation direction into link style. */
    public static int direct2style( boolean directed )
    {
	int style = PLAIN;
        if ( directed )
            style = ARROW;
        else
            style = PLAIN;
	return style;
    }

    /*
    // Return the ith concept node in the relation link.
    public ConceptNode getNode( int i )
    {
	return ( ConceptNode )concepts.get(i);
    }
    */

    /** Returns the 1st concept in the relation. */
    public ConceptNode getNode1()
    {
	return ( ConceptNode )concepts.get(0);
    }

    /** Returns the 2nd concept in the relation. */
    public ConceptNode getNode2()
    {
	return ( ConceptNode )concepts.get(1);
    }
    	
    /** Draws an arrow from point P1(x1, y1) to point P2(x2, y2).
	@param w the arrow end width;
	@param l the ratio between arrow end length and line length.
    */
    public static void drawArrow( Graphics g, int x1, int y1, int x2, int y2, int w, double l )
    {
	// distance between P1 and P2
	double d = Math.sqrt( (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) );

	// norm N perpendicular to vector P1P2. 
	double xn = ( y1 - y2 ) / d;
	double yn = ( x2 - x1 ) / d;

	// projection Po of arrow end on P1P2.
	double xo = l*x1 + (1-l)*x2;
	double yo = l*y1 + (1-l)*y2;

	// arrow end point Pa1 and Pa2
	int xa1 = (int)( xo + w * xn );
	int ya1 = (int)( yo + w * yn );
	int xa2 = (int)( xo - w * xn );
	int ya2 = (int)( yo - w * yn );			 

	g.drawLine( xa1, ya1, x2, y2 );
	g.drawLine( xa2, ya2, x2, y2 );
    }
	
    /** Paints the object. */
    public void paint( Graphics2D g )
    {
        /*
	   Point ps = relation.concepts.get(0).getCenter();
	   Point pe = relation.concepts.get(1).getCenter();
	   int xs = ns.getCenterX();
	   int ys = ns.getCenterY();
	   int xe = ne.getCenterX();
	   int ye = ne.getCenterY();
	*/

	ConceptNode ns = getNode1();
 	ConceptNode ne = getNode2();
	double d = ns.distance(ne);
	double t1 = ns.size/2 / d;
	double t2 = ne.size/2 / d;
	int x1 = ( int )( ( 1 - t1 ) * ns.x + t1 * ne.x );
	int y1 = ( int )( ( 1 - t1 ) * ns.y + t1 * ne.y );
	int x2 = ( int )( ( 1 - t2 ) * ne.x + t2 * ns.x );
	int y2 = ( int )( ( 1 - t2 ) * ne.y + t2 * ns.y );

	Color c = new Color( color.getRed(), color.getGreen(), color.getBlue(), alpha );
        g.setColor( c );
        g.setStroke( new BasicStroke( width, BasicStroke.CAP_BUTT, 0 ) );
	g.drawLine( x1, y1, x2, y2 );
	if ( style == ARROW ) 
	    drawArrow( g, x1, y1, x2, y2, width+2, 0.1 ); 
	/*
	g.drawLine( ns.getCenterX(), ns.getCenterY(), ne.getCenterX(), ne.getCenterY() );
	if ( isDirected() ) 
	    drawArrow( g, ns.getCenterX(), ns.getCenterY(), ne.getCenterX(), ne.getCenterY(), width+2, 0.1 ); 
	*/
    }
    
    /** String representation.    */
    public String toString()
    {
	String str = super.toString() + "\n";
	str += " ==> Link: \t" + color + "\t" + alpha + "\t";		
	if ( style == PLAIN )
	    str +=  "PLAIN";
	else
	    str += "ARROW";	
	return str;
    }
}
