///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST++
// Ying Feng
// Computer Scicence, Indiana University
// Feb-Aug 2003
//
// Class Test45a
//   The class to run Test 45a for ABSURDIST.
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



package test;

import java.io.*; 

import absurdist.*;


public class Test45a extends TestBase
{

    int nLabel;
    int directMode;
    boolean weighted;
       

  /** Creates the base system. Derived classes may redefine
	this method according to how they need to create it.
    */
      
    ConceptSystem getSysO() throws Exception {
	return new ConceptSystem( n, 1, false, false, 
				  fullness, nLabel, directMode, weighted );
    }

    /** Creates the noisy system. Derived classes may redefine
	this method according to how they need to create it.
    */
    ConceptSystem getSysN( ConceptSystem systemO)  throws Exception{
	return     new ConceptSystem( systemO, 0, 0, noise, noise );
    }

   /** Fills the external similarity matrix. By default, it is
	filled with all zeros, but derived classes implementing 
	experiments with external similarity may set some non-zero
	values.
     */
    void  seedExSim(Absurdist absurdist) {
 	// initialize the external similarity matrix to all 0's
	absurdist.initExSim();
	absurdist.alpha = 1.0;
	
	// seed the external similarity matrix
	for ( int io=0; io<n;  io++ ) {
	    for ( int in=0; in<n; in++ ) 
		absurdist.exSim[io][in] = 0;
	}
	int idx = ( int )( Math.random() * n ); // randomly choose a concept pair to seed
	absurdist.exSim[idx][idx] = 1;	    
    }



    public static void main( String[] args )
    {
        if ( args.length < 2 ) {
            error( "Please input number of concepts and noise" );
            return;
        }
	Test45a test=new Test45a();
	test.mainCall( args );
    }

    void mainCall(String[] args ) {
  
        try {
            int n = Integer.parseInt( args[0] );
            double fullness = 0.5;
	    nLabel = Integer.parseInt( args[1] );
	    directMode = Integer.parseInt( args[2] );
	    weighted = args[3].equals( "w" ) ? true : false;
            double noise = 0.02;

	    String ls = "l" + nLabel;
	    String ds = "d" + directMode;
	    String ws = weighted ? "weighted" : "unweighted";
	    String str = ls + ds + args[3];

	    String testDir = Absurdist.prop.getOption("testDir", ".");
	    String fname = testDir + "/test45a/n" +args[0] + str + ".45a";
	    PrintStream out = new PrintStream( new FileOutputStream(fname));
	    //PrintStream out = new PrintStream( new FileOutputStream( "n" + args[0] + str + ".45a" ) );
	
	    out.println( "Test 45a: N = " + n + ", nLabel = " + nLabel + 
			 ", directed = " + directMode + ", " + ws +
			 ", noise = " + (noise*100) + "%");

	    //NRuns = 200;
	    mainLoop( n, fullness, noise, NRuns, out);


        }
        catch ( Exception e ) {
            e.printStackTrace();
            System.exit(0);
        }
    }
}
