///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST++
// Ying Feng
// Computer Scicence, Indiana University
// Feb-Aug 2003
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



package test;

import java.io.*; 

import absurdist.*;
import common.CSGI;



/** Class Test45c
   The class to run Test 45c or ABSURDIST.
*/
public class Test45c extends TestBase
{
 
    int nLabel;
    int directMode;
    int smode;
    boolean weighted;
    double alpha;     


 /** Creates the base system. Derived classes may redefine
	this method according to how they need to create it.
    */
      
    ConceptSystem getSysO() throws Exception {
	return  new ConceptSystem( n, 1, false, false, 
				   fullness, nLabel, directMode, weighted );


    }

    /** Creates the noisy system. Derived classes may redefine
	this method according to how they need to create it.
    */
    ConceptSystem getSysN( ConceptSystem systemO)  throws Exception{
	return new ConceptSystem( systemO, 0, 0, noise, noise );
    }

   /** Fills the external similarity matrix and sets alpha. By default, it is
	filled with all zeros, but derived classes implementing 
	experiments with external similarity may set some non-zero
	values.
     */
    void  seedExSim(Absurdist absurdist) {
 	// initialize the external similarity matrix to all 0's
	absurdist.initExSim();
	absurdist.alpha = alpha;
	// Convert to CSGI, to compute node degrees
	CSGI csO = new CSGI(systemO);
	int idxMax = csO.getMaxDgrConceptIdx();
	int idxMin = csO.getMinDgrConceptIdx();
	out.println("Degree range: deg("+idxMin+")="+csO.degree(idxMin)
		    +", deg("+idxMax+")="+csO.degree(idxMax));
	
	int idx=0;
	if ( smode == 0 )
	    idx = ( int )( Math.random() * n ); // randomly choose a concept pair to seed
	else if ( smode == 1 )
	    idx = idxMax;
	else if ( smode == 2 )
	    idx = idxMin;
	else {
	    throw new AssertionError("Invalid smode="+smode);
	}
	absurdist.exSim[idx][idx] = 1; 		
   }



    public static void main( String[] args )
    {
        if ( args.length != 4 ) {
            String msg = 
		"Usage: java Test7 smode nLabel directMode weighted\n" +
		" smode = seeding mode (0|1|2) \n" +
		" nLabel = number of labels (1 or more)\n" +
		" directMode = 0|1 \n" +
		" weighted = w|n";
            error( msg);
            return;
        }
        
	Test45c test=new Test45c();
	test.mainCall( args );
    }

    void mainCall(String[] args ) {
        try {
            int n = 12;
            double fullness = 0.5;
            alpha = 1.0; //// Integer.parseInt( args[0] )/( double )10;
	    /** seeding mode: 
		<pre>
		0: random; 
		1: node with maximum degree; 
		2: node with minimum degree
		</pre>
	    */
            smode = Integer.parseInt( args[0] );
	    nLabel = Integer.parseInt( args[1] );
	    directMode = Integer.parseInt( args[2] );
	    weighted = args[3].equals( "w" ) ? true : false;
            double noise = 0.02;

	    String ls = "l" + nLabel;
	    String ds = "d" + directMode;
	    String ws = weighted ? "weighted" : "unweighted";
	    String str = ls + ds + args[3];


	    String testDir = Absurdist.prop.getOption("testDir", ".");
	    String fname = testDir + "/test45a/m" +args[0] + str + ".45c";
	    PrintStream out = new PrintStream(new FileOutputStream( fname));

	    //PrintStream out = new PrintStream( new FileOutputStream( "m" + args[0] + str + ".45c" ) );
	    out.println( "Test 45c: smode = " + smode + ", nLabel = " + nLabel + ", directed = " + directMode + ", " + ws );
	    out.println( "\nindex\tmisCon\tmisRel\taccuracy" );
	    out.println( "----------------------------------------" );

	    // test on 200 random systems
	    int count = 0;          // count of optimal mappings
	    double avgMisCon = 0;   // average mismapped concepts 
	    double avgMisRel = 0;   // average mismapped relations
	    double avgAccuracy = 0; // average mapping accuracy 


	    //NRuns = 200;
	    mainLoop( n, fullness, noise, NRuns, out);

        }
        catch ( Exception e ) {
            e.printStackTrace();
            System.exit(0);
        }
    }
}
