///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST II
// Ying Feng
// Computer Scicence, Indiana University
// Feb-Aug 2003
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



package test;

import java.io.*; 

import absurdist.*;
import common.*;


/** Class TestExSimMode
    runs Test 4a on pre-seeding on external similiarity matrix on simple graph
    (each parameter setting is tested with 100 runs).
    Fixed: 
      maxLoop = 3000, L = 0.01
      Graph type = simple asymmetric graph (unlabeled, undirected, unweighted)
      Concept number = 12
      Relation fullness = 50%           
      Seed Num = 1
    Varying:
      Seed Mode = none, rand, max, min
      Noise level = 0, 2%, 4%, 6%, 8%
    Note: 
      All seeded concept pairs are correct mappings;
      Seeding mode refers to whether to choose none concept, a random concept, 
      a concept with maximum or minimum degree to pre-map.
      Seeding number refers to how many pairs of concepts are being seeded. 
*/
public class TestExSimMode extends TestBase
{
    
    String smode;

  /** Fills the external similarity matrix. By default, it is
	filled with all zeros, but derived classes implementing 
	experiments with external similarity may set some non-zero
	values.
     */
    void  seedExSim(Absurdist absurdist) {
	absurdist.alpha = 1.0;
	// initialize the external similarity matrix to all 0's
	absurdist.initExSim();

	CSGI sysO = new CSGI( systemO );
	int idx = 0;
	if ( smode.equals( "rand" ) ) // randomly choose a concept pair to seed
	    idx = ( int )( Math.random() * n );	
	else if ( smode.equals( "max" ) ) // choose a concept with maximum degree
	    idx = sysO.getMaxDgrConceptIdx(); 
	else if ( smode.equals( "min" ) ) // choose a concept with minimum degree
	    idx = sysO.getMinDgrConceptIdx(); 
	absurdist.exSim[idx][idx] = 1; 
    }
 
    public static void main( String[] args )
    {
        if ( args.length < 2 ) {
            error( "Please input number of concepts and noise" );
            return;
        }
        
	TestExSimMode test=new TestExSimMode();
	test.mainCall( args );
    }




    void mainCall(String[] args ) {

       try {
            int n = 12;
            double fullness = 0.5;
            smode = args[0].toLowerCase();
            double noise = Double.parseDouble( args[1] ) / 100;

	    String testDir = Absurdist.prop.getOption("testDir", ".");
	    String fname = testDir + "/test4a/" + smode + "n" + args[1] + ".t4a";
	    PrintStream out = new PrintStream( new FileOutputStream( fname ) );
	    out.println( "Test 4a: seed mode = " + smode + ", noise = " + args[1] + "%" );
	    mainLoop( n, fullness, noise, NRuns, out);

        }
        catch ( Exception e ) {
            e.printStackTrace();
            System.exit(0);
        }
    }
}
