///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST II
// Ying Feng
// Computer Scicence, Indiana University
// Feb-Aug 2003
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



package test;

import java.io.*; 

import absurdist.*;
import common.*;


/** Class TestExSimNum
    runs Test 4b on pre-seeding on external similiarity matrix on simple graph
    (each parameter setting is tested with 100 runs).
    Fixed: 
      maxLoop = 3000, L = 0.01
      Graph type = simple asymmetric graph (unlabeled, undirected, unweighted)
      Concept number = 20
      Relation fullness = 50% 
      Seed Mode = rand
    Varying:
      Seed Num = 0, 1, 2, 3, 4
      Noise level = 0, 2%, 4%, 6%, 8%
    Note: 
      All seeded concept pairs are correct mappings;
      Seeding mode refers to whether to choose none concept, a random concept, 
      a concept with maximum or minimum degree to pre-map.
      Seeding number refers to how many pairs of concepts are being seeded. 
*/
public class TestExSimNum extends TestBase
{
  
    // controls the number of non-zeros in the external similarity matrix
    int snum;

    /** Fills the external similarity matrix. By default, it is
	filled with all zeros, but derived classes implementing 
	experiments with external similarity may set some non-zero
	values.
     */
    void  seedExSim(Absurdist absurdist) {
 	// initialize the external similarity matrix to all 0's
	absurdist.initExSim();
	absurdist.alpha = 1.0;

	// seed the external similarity matrix
	for ( int num = 0; num < snum; num++ ) {
	    int idx = 0;
	    do {
		idx = ( int )( Math.random() * n );
	    }	
	    while ( absurdist.exSim[idx][idx] == 1 );		    
	    absurdist.exSim[idx][idx] = 1; 
	}
    }

    public static void main( String[] args )
    {
        if ( args.length < 2 ) {
            error( "Please input number of concepts and noise" );
            return;
        }
        
	TestExSimNum test=new TestExSimNum();
	test.mainCall( args );
    }

    void mainCall(String[] args ) {

        try {
            int n = 12;
            double fullness = 0.5;
	    snum = Integer.parseInt( args[0] );
            double noise = Double.parseDouble( args[1] ) / 100;

	    String testDir = Absurdist.prop.getOption("testDir", ".");
	    String fname = testDir + "/test4b/s" + snum + "n" + args[1] + ".t4b";
	    PrintStream out = new PrintStream( new FileOutputStream( fname ) );
	    out.println( "Test 4b: seed num = " + args[0] + ", noise = " + args[1] + "%" );

	    mainLoop( n, fullness, noise, NRuns, out);

        }
        catch ( Exception e ) {
            e.printStackTrace();
            System.exit(0);
        }
    }
}
