///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST II
// Ying Feng
// Computer Scicence, Indiana University
// Feb-Aug 2003
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



package test;

import java.io.*; 

import absurdist.*;


/** Class TestNoiseRCRI
    runs Test 3 on RC/RI noise tolerance on weighted graph (each parameter setting is tested with 100 runs).
    Fixed: 
      was: maxLoop = 3000, L = 0.01
      Graph type = unlabeled undirected weighted graph
      Concept number = 12
      Relation fullness = 50%           
    Varying:
      Noise mode = rc, ri, rcri
      RC/RI Noise level = 0, 2%, 4%, 6%, 8%
*/  
public class TestNoiseRCRI extends TestBase
{
      
    /** Creates the base system. Redefined by this class.
    */
      
    ConceptSystem getSysO() throws Exception  {
	return new ConceptSystem( n, 1, false, false, fullness, 1, 0, true );
    }

    /** Creates the noisy system. Redefined by this class.
    */
    ConceptSystem getSysN( ConceptSystem systemO)  throws Exception  {
	return new ConceptSystem( systemO, 0, 0, rcNoise, riNoise );
    }

    
    double rcNoise = 0, riNoise = 0;
  
    public static void main( String[] args )
    {
        if ( args.length < 2 ) {
            error( "Please input number of concepts and noise" );
            return;
        }
    
	TestNoiseRCRI test=new TestNoiseRCRI();
	test.mainCall( args );
    }

    void mainCall(String[] args ) {

        try {
            int n = 12;
            double fullness = 0.5;
            String mode = args[0].toLowerCase();
            double noise = Double.parseDouble( args[1] ) / 100;
	    
	    if ( mode.equals( "rc" ) )
		rcNoise = noise;
	    else if ( mode.equals( "ri" ) )
		riNoise = noise;
	    else if ( mode.equals( "rcri" ) ) 
		rcNoise = riNoise = noise;	  

	    String testDir = Absurdist.prop.getOption("testDir", ".");
	    String fname = testDir + "/test3/" + mode + "n" + args[1] + ".t3";
	    PrintStream out = new PrintStream( new FileOutputStream( fname ) );
	    out.println( "Test 3: noiseMode = " + mode + ", rcNoise = " + rcNoise + ", riNoise = " + riNoise );


	    //debugAll = true; NRuns=1;
	    mainLoop( n, fullness, noise, NRuns, out);

        }
        catch ( Exception e ) {
            e.printStackTrace();
            System.exit(0);
        }
    }
}
