///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST II
// Ying Feng
// Computer Science, Indiana University
// Feb-Aug 2003
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


package driver;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import absurdist.*;
import graph.*;
import gui.*;
import penalty.*;


/**
    Class AppletDriver
    is the Applet to run ABSURDIST II in an interactive GUI.
*/

public class AppletDriver extends JApplet implements ActionListener, MouseInputListener
{
    // mode for debug message
    static boolean debugMode = true; 

    // mapping algorithm modes
    /**  absurdist-original
    */
    // static final int ABS0 = 0;
    /**  absurdist-Jul-17-2003: Sum(alpha*R-chi*I)=0
    */
    static final int ABS1 = 0;
    /**  exponential
    */
    static final int EXP = 1;
    /**  penalty
    */
    static final int PEN = 2;

    // refresh the screen every 100 milliseconds
    static final int DT = 100;	

    // system matchers
    Absurdist absurdist; 	// absurdist 
    PenaltyMinimizer penalmin;  // penalty minimizer
    
    /*
    Image offscreenImage;	// the offscreen bitmap
    Graphics offscreenGraphics; // its graphics context  
    int width, height; 		// this applet's dimensions
    Color graphColor;	        // graph background color
    Color plotColor;	        // plot background color
    */
    
    // control components
    //// JLabel nameLabel;      // label for concept graph name
    //// JTextField nameField;	// text input for concept graph name

    // controls for generating random new system 
    JPanel genPanel;            // subpanel to hold the following controls 
    // concept parameters
    JLabel nConLabel;		// label for number of concepts
    JTextField nConField;	// text input for number of concepts
    JLabel nCatLabel;		// label for number of concept categories
    JTextField nCatField;	// text input for number of concept categories
    JCheckBox rankCheck;	// check box to control whether concepts are ranked
    JCheckBox valueCheck;	// check box to control whether concepts are valued 
    // relation parameters
    JLabel fullLabel;		// label for fullness of relations
    JSlider fullSlider;		// slider to control fullness of relations
    JLabel nLabelLabel;		// label for number of relation labels/models
    JTextField nLabelField;	// text input for number of relation labels/models
    JComboBox directCombo;	// combo box to choose relation direction mode (directed/undirected/mixed)
    JCheckBox weightCheck;	// check box to control whether relations are weighted
    JButton genButton;	        // button to generate random concept system
    //// JCheckBox asymCheck;	// check box to control whether concept graph is asymmetric

    // controls for copying system with added noise
    JPanel copyPanel;           // subpanel to hold the following controls 
    JLabel ccnLabel;		// label for concept coverage noise
    JSlider ccnSlider;	        // slider to control concept coverage noise
    JLabel cinLabel;		// label for concept intensity noise
    JSlider cinSlider;	        // slider to control concept intensity noise
    JLabel rcnLabel;		// label for relation coverage noise
    JSlider rcnSlider;	        // slider to control relation coverage noise
    JLabel rinLabel;		// label for relation intensity noise
    JSlider rinSlider;	        // slider to control relation intensity noise
    JButton copyButton;	        // button to copy concept system with noise

    // controls for mapping systems    
    JPanel mapPanel;            // subpanel to hold the following controls 
    JComboBox algCombo;	        // combo box to choose mapping algorithms (absurdist/exponential/penalty)
    JLabel alphaLabel;          // label for external similarity coefficient
    JSlider alphaSlider;        // slider to control external similarity coefficient
    JLabel betaLabel;           // label for activator coefficient
    JSlider betaSlider;         // slider to control activator coefficient
    JButton mapButton;		// button to map systems
    
    // display panels
    GraphPanel graphPanel;	// panel to display concept graphs
    PlotPanel plotPanel;	// panel for variable plots
    
    // process status
    JLabel statusLabel;		// label to show mapping status

    // grid bag layout/constraints used by sub-panels in control box
    GridBagLayout gbLayout; 
    GridBagConstraints gbConstr;

    // Initialize GUI components.
    public void init() 
    {
	/*
        absurdist = new Absurdist();
	penalmin = new PenaltyMinimizer();        
        */

	//setPreferredSize( new Dimension( 1000, 800 ) );
	//setSize( 500, 500 );
	//setLocation( 0, 0 );            

	// make sure sliders are not too big
	Dimension sliderDim = new Dimension(70,8);

	Container contentPane = getContentPane();
	contentPane.setLayout( new BorderLayout() );
        
	// control box containing components 	
	Box controlBox = Box.createHorizontalBox();
        controlBox.setSize( getWidth(), 100 );
	controlBox.add( Box.createHorizontalStrut(1) );
	contentPane.add( controlBox, BorderLayout.NORTH );
	//// InputHandler inputHandler = new InputHandler();

	/*
        nameLabel = new JLabel( "Sys name" );
        controlBox.add( nameLabel );        
        nameField = new JTextField(8);
        //// numField.addActionListener( this );
        controlBox.add( nameField );
	*/

	// init grid bag layout/constraints used by sub-panels in control box
	gbLayout = new GridBagLayout(); 
	gbConstr = new GridBagConstraints();
	gbConstr.fill = GridBagConstraints.BOTH;
	gbConstr.weighty = 1;

	// init controls for generating random new system 
	genPanel = new JPanel();
	genPanel.setLayout( gbLayout );
	controlBox.add( genPanel );	

        nConLabel = new JLabel( "Concept #" );
        addGridComp( nConLabel, genPanel, 0, 0, 1, 1, 0 );        
        nConField = new JTextField( "8", 3 );
	addGridComp( nConField, genPanel, 0, 1, 1, 1, 1 );

        nCatLabel = new JLabel( "Category #" );
        addGridComp( nCatLabel, genPanel, 0, 2, 1, 1, 0 );        
        //// addGridComp( nCatLabel, genPanel, 1, 0, 1, 1, 0 );        
        nCatField = new JTextField( "3", 2 );
        addGridComp( nCatField, genPanel, 0, 3, 1, 1, 0.5 );
        //// addGridComp( nCatField, genPanel, 1, 1, 1, 1, 1 );
	
        rankCheck = new JCheckBox( "Ranked" );
        addGridComp( rankCheck, genPanel, 0, 4, 1, 1, 0 );      
        //// addGridComp( rankCheck, genPanel, 2, 0, 2, 1, 0 );      

        valueCheck = new JCheckBox( "Valued" );
        addGridComp( valueCheck, genPanel, 0, 5, 1, 1, 0 );      
        //// addGridComp( valueCheck, genPanel, 3, 0, 2, 1, 0 );      
	
        fullLabel = new JLabel( "Relation %" );
        addGridComp( fullLabel, genPanel, 1, 0, 1, 1, 0 );        
        //// addGridComp( fullLabel, genPanel, 0, 2, 1, 1, 0 );        
        fullSlider = new JSlider();
	fullSlider.setPreferredSize(sliderDim);
        addGridComp( fullSlider, genPanel, 1, 1, 1, 1, 1 );        
        //// addGridComp( fullSlider, genPanel, 0, 3, 1, 1 );        
 	
        nLabelLabel = new JLabel( "Label #" );
        addGridComp( nLabelLabel, genPanel, 1, 2, 1, 1, 0 );        
        //// addGridComp( nLabelLabel, genPanel, 1, 2, 1, 1, 0 );        
        nLabelField = new JTextField( "3", 2 );
        addGridComp( nLabelField, genPanel, 1, 3, 1, 1, 0.5 );
        //// addGridComp( nLabelField, genPanel, 1, 3, 1, 1, 1 );
	
	String[] directModes = { "Undirected", "Directed", "Mixed" };
        directCombo = new JComboBox( directModes );
        addGridComp( directCombo, genPanel, 1, 4, 1, 1, 0 );
        //// addGridComp( directCombo, genPanel, 2, 2, 2, 1, 0 );

	weightCheck = new JCheckBox( "Weighted" );
        addGridComp( weightCheck, genPanel, 1, 5, 1, 1, 0 );
        //// addGridComp( weightCheck, genPanel, 3, 2, 2, 1, 0 );

        genButton = new JButton( "Generate Random System" );
        genButton.addActionListener( this );
        addGridComp( genButton, genPanel, 3, 0, 6, 1, 0 );
        //// addGridComp( genButton, genPanel, 4, 0, 4, 1, 0 );
	
	/*
        asymCheck = new JCheckBox( "Asym" );
        //// fullSlider.addChangeListener( this );
        controlBox.add( asymCheck );
	*/

	// init controls for copying system with added noise
	copyPanel = new JPanel();
	copyPanel.setLayout( gbLayout );
	controlBox.add( Box.createHorizontalStrut(3) );
	controlBox.add( copyPanel );	
	//// controlBox.add( Box.createHorizontalStrut(1) );
	
        ccnLabel = new JLabel( "CC Noise" );
        addGridComp( ccnLabel, copyPanel, 0, 0, 1, 1, 0 );         
        ccnSlider = new JSlider();
	ccnSlider.setPreferredSize(sliderDim);
        addGridComp( ccnSlider, copyPanel, 0, 1, 1, 1, 1 );         
        
        cinLabel = new JLabel( "CI Noise" );
        addGridComp( cinLabel, copyPanel, 0, 2, 1, 1, 0 );         
	//// addGridComp( cinLabel, copyPanel, 1, 0, 1, 1, 0 );         
	cinSlider = new JSlider();
	cinSlider.setPreferredSize(sliderDim);
        addGridComp( cinSlider, copyPanel, 0, 3, 1, 1, 1 );         
        //// addGridComp( cinSlider, copyPanel, 1, 1, 1, 1, 1 );         
        
        rcnLabel = new JLabel( "RC Noise" );
        addGridComp( rcnLabel, copyPanel, 1, 0, 1, 1, 0 );         
        //// addGridComp( rcnLabel, copyPanel, 2, 0, 1, 1, 0 );         
        rcnSlider = new JSlider();
	rcnSlider.setPreferredSize(sliderDim);
        addGridComp( rcnSlider, copyPanel, 1, 1, 1, 1, 1 );         
        //// addGridComp( rcnSlider, copyPanel, 2, 1, 1, 1, 1 );         
        
        rinLabel = new JLabel( "RI Noise" );
        addGridComp( rinLabel, copyPanel, 1, 2, 1, 1, 0 );         
        //// addGridComp( rinLabel, copyPanel, 3, 0, 1, 1, 0 );         
        rinSlider = new JSlider();
	rinSlider.setPreferredSize(sliderDim);
        addGridComp( rinSlider, copyPanel, 1, 3, 1, 1, 1 );         
        //// addGridComp( rinSlider, copyPanel, 3, 1, 1, 1, 1 );         
        
        copyButton = new JButton( "Copy Noisy System" );
        copyButton.addActionListener( this );
        addGridComp( copyButton, copyPanel, 2, 0, 4, 1, 0 ); 	        
        //// addGridComp( copyButton, copyPanel, 4, 0, 2, 1, 0 ); 	        

	// init controls for mapping systems    
	mapPanel = new JPanel();
	mapPanel.setLayout( gbLayout );
	controlBox.add( Box.createHorizontalStrut(1) );
	controlBox.add( mapPanel );	
	//// controlBox.add( Box.createHorizontalStrut(1) );

	String[] algModes = { /*"Absurdist0",*/ "Absurdist1", "Exponential", "Penalty" };
        algCombo = new JComboBox( algModes );
        addGridComp( algCombo, mapPanel, 0, 0, 1, 2, 0 );
        //// addGridComp( algCombo, mapPanel, 0, 0, 1, 1, 0 );

        alphaLabel = new JLabel( "External Coeff" );
        addGridComp( alphaLabel, mapPanel, 0, 1, 1, 1, 0 );        
        //// addGridComp( alphaLabel, mapPanel, 1, 0, 1, 1, 0 );        
        alphaSlider = new JSlider();
	alphaSlider.setPreferredSize(sliderDim);
        addGridComp( alphaSlider, mapPanel, 0, 2, 1, 1, 1 );        
        //// addGridComp( alphaSlider, mapPanel, 1, 1, 1, 1, 1 );        

        betaLabel = new JLabel( "Activator Coeff" );
        addGridComp( betaLabel, mapPanel, 1, 1, 1, 1, 0 );        
        //// addGridComp( betaLabel, mapPanel, 2, 0, 1, 1, 0 );        
	betaSlider = new JSlider();
	betaSlider.setPreferredSize(sliderDim);
        addGridComp( betaSlider, mapPanel, 1, 2, 1, 1, 1 );        
        //// addGridComp( betaSlider, mapPanel, 2, 1, 1, 1, 1 );        

        mapButton = new JButton( "Map Systems" );
        mapButton.addActionListener( this );
        addGridComp( mapButton, mapPanel, 2, 0, 3, 1, 0 ); 	        
        //// addGridComp( mapButton, mapPanel, 3, 0, 2, 1, 0 ); 	        

        // display panel for graph mapping
        //// graphPanel = new JGraphPanel( absurdist );
	graphPanel = new GraphPanel();
	graphPanel.setBackground( Color.black );
	graphPanel.setPreferredSize( new Dimension( getWidth()/2, getHeight()-200 ) );
        graphPanel.addMouseListener( this );
        contentPane.add( graphPanel, BorderLayout.WEST );                

        // display panel for plots
        //// plotPanel = new JPlotPanel( new GridLayout( 1, 4 ) );
        //// plotPanel = new JPlotPanel( absurdist );
	plotPanel = new PlotPanel();
        plotPanel.setBackground( Color.white );
	plotPanel.setPreferredSize( new Dimension( getWidth()/2, getHeight()-200 ) );
        contentPane.add( plotPanel, BorderLayout.EAST );                

        /*
        plotRPanel = new JPlotPanel();
        plotRPanel.setSize( plotPanel.getWidth(), plotPanle.getHeight()/4 );
        plotPanel.add( plotRPanel );
        plotIPanel = new JPlotPanel();
        plotIPanel.setSize( plotPanel.getWidth(), plotPanel.getHeight()/4 );
        plotPanel.add( plotIPanel );
        plotNPanel = new JPlotPanel();
        plotNPanel.setSize( plotPanel.getWidth(), plotPanel.(getHeight()/4 );
        plotPanel.add( plotNPanel );
        plotCPanel = new JPlotPanel();
        plotCPanel.setSize( plotPanel.getWidth(), plotPanel.(getHeight()/4 );
        plotPanel.add( plotCPanel );
        */
    
        statusLabel = new JLabel( "Initialized" );
        statusLabel.setSize( getWidth(), 100 );
        contentPane.add( statusLabel, BorderLayout.SOUTH );
	
	//// debugln( "Graph panel now ( " + graphPanel.getWidth() + ", " + graphPanel.getHeight() + " )" );
	//// debugln( "Plot panel now ( " + plotPanel.getWidth() + ", " + plotPanel.getHeight() + " )" );

	/*
        //create the offscreen bitmap
        buffer = this.createImage( getWidth, getHeight );
        */
     }

    // Add components to a container using grid layout.
    private void addGridComp( Component component, Container container, 
			      int row, int column, int width, int height, double weightx )
    {
	gbConstr.gridx = column;
	gbConstr.gridy = row;
	gbConstr.gridwidth = width;
	gbConstr.gridheight = height;
	gbConstr.weightx = weightx;
	gbLayout.setConstraints( component, gbConstr );
	container.add( component );
    }       
	
    // Action listener for text fields and buttons.
    public void actionPerformed( ActionEvent e )
    {     
	/*
	debugln( "\ngenPanel: " + genPanel.getX() + " " + genPanel.getY() + " " 
		 + genPanel.getWidth() + " " + genPanel.getHeight() );
	debugln( "copyPanel: " + copyPanel.getX() + " " + copyPanel.getY() + " " 
		 + copyPanel.getWidth() + " " + copyPanel.getHeight() );
	debugln( "mapPanel: " + mapPanel.getX() + " " + mapPanel.getY() + " " 
		 + mapPanel.getWidth() + " " + mapPanel.getHeight() );
	*/

        if ( e.getSource() == genButton ) { // generate absurdist system with random graph
	    //// String name = nameField.getText();
	    //// String name = "Random";

	    // concept parameters
            int n = Integer.parseInt( nConField.getText() );
	    int nCat = Integer.parseInt( nCatField.getText() );
	    boolean ranked = rankCheck.isSelected();
	    boolean valued = valueCheck.isSelected();

	    // relation parameters
            double fullness = ( double )fullSlider.getValue() / 200; // [ 0, 50% ]
	    int nLabel = Integer.parseInt( nLabelField.getText() );
            int directMode = directCombo.getSelectedIndex();
	    boolean weighted = weightCheck.isSelected();

	    // noise parameters
            double noiseCC = (double)ccnSlider.getValue() / 1000; // [ 0, 10% ]
	    double noiseCI = (double)cinSlider.getValue() / 1000; // [ 0, 10% ]
            double noiseRC = (double)rcnSlider.getValue() / 1000; // [ 0, 10% ]
	    double noiseRI = (double)rinSlider.getValue() / 1000; // [ 0, 10% ]
	    
            //// boolean asymmetric = asymCheck.isSelected();
	    //// boolean useAbsd = absdRadio.isSelected();

	    debugln( "\nParameters for generating random system:" );
	    debugln( "n = " + n + ", nCat = " + nCat + ", ranked = " + ranked + ", valued = " + valued );
	    debugln( "fullness = " + fullness + ", nLabel = " + nLabel + 
		     ", directMode = " + directMode + ", weighted = " + weighted );
	
	    ConceptGraph graphO, graphN;
	    try {
		ConceptSystem systemO = new ConceptSystem( n, nCat, ranked, valued, 
							   fullness, nLabel, directMode, weighted );
		graphO = new ConceptGraph( systemO, 0,0, graphPanel.getWidth(),
					   graphPanel.getHeight()/2 );
		ConceptSystem systemN = new ConceptSystem( systemO, noiseCC, noiseCI, noiseRC, noiseRI );
		graphN = new ConceptGraph( systemN,0,graphPanel.getHeight()/2, 
					   graphPanel.getWidth(),graphPanel.getHeight()/2 );
	    } 
	    catch ( Exception ex ) {
		Alert.alert(ex.getMessage());
		ex.printStackTrace();		
		return;
	    }

	    try {
		absurdist = new Absurdist( graphO, graphN );
		absurdist.printSystems();
		/*
		ConceptSystem systemN = new ConceptSystem( systemO );
		systemN.addNoise( noiseCC, noiseCI, noiseRC, noiseRI );
		absurdist.setSystem( graphO, graphN );
		*/
	    }
	    catch ( Exception ex ) {
		ex.printStackTrace();
		System.exit(-1);
	    }	    

	    graphPanel.setAbsurdist( absurdist );
	    plotPanel.setAbsurdist( absurdist );
            graphPanel.repaint();
            plotPanel.repaint();
        }
	/*
        if ( e.getSource() == copyButton ) { // copy system with added noise
	    // noise parameters
            double noiseCC = ( double )ccnSlider.getValue() / 1000; // [ 0, 10% ]
	    double noiseCI = ( double )cinSlider.getValue() / 1000; // [ 0, 10% ]
            double noiseRC = ( double )rcnSlider.getValue() / 1000; // [ 0, 10% ]
	    double noiseRI = ( double )rinSlider.getValue() / 1000; // [ 0, 10% ]

	    debugln( "Noise when copying system: " );
	    debugln( "noiseCC = " + noiseCC + ", noiseCI = " + noiseCI + 
		     ", noiseRC = " + noiseRC + ", noiseRI = " + noiseRI );
	    try {
		ConceptSystem systemN = new ConceptSystem( systemO, noiseCC, noiseCI, noiseRC, noiseRI );
		ConceptGraph graphN = new ConceptGraph( systemN, 0, graphPanel.getHeight()/2, 
							graphPanel.getWidth(), graphPanel.getHeight()/2 );
		absurdist.setSystemN( graphN );
		absurdist.printSystem( null );
	    }
	    catch ( Exception ex ) {
		ex.printStackTrace();
		System.exit(-1);
	    }	    

	    graphPanel.setAbsurdist( absurdist );
	    plotPanel.setAbsurdist( absurdist );
            graphPanel.repaint();
            plotPanel.repaint();
        }
	*/
        else if ( e.getSource() == mapButton ) { // map concept systems 
	    debugln( "Map systems" );

	    // disable these buttons until the computation thread finishes
	    genButton.setEnabled( false );
	    copyButton.setEnabled( false );
	    mapButton.setEnabled( false );

	    // start animation
	    AnimationThread animThread = new AnimationThread();
	    if ( animThread != null ) { // ??? don't need this
		animThread.start();  
		debugln( "Main thread: Animation thread has been spawned off." );
	    }

	    // start computation
	    ComputationThread compThread = new ComputationThread( animThread );
	    compThread.start();
	    debugln( "Main thread: Computation thread has been spawned off." );
        }
    }
        
    public void mouseClicked( MouseEvent e )
    {    
    }
        
    public void mouseEntered( MouseEvent e )
    {    
    }        

    public void mouseExited( MouseEvent e )
    {    
    }        

    public void mousePressed( MouseEvent e )
    {    
    }        

    public void mouseReleased( MouseEvent e )
    {    
    }  

    public void mouseDragged( MouseEvent e )
    {    
    }  

    public void mouseMoved( MouseEvent e )
    {    
    } 

    public static void debug( String s )
    {
        if ( debugMode )
            System.out.print( s );
    }
        
    public static void debugln( String s )
    {
        if ( debugMode )
            System.out.println( s );
    }
        
    public static void error( String s )
    {
        //// if ( errorMode )
        System.err.println( s );
    }              


    // Thread to monitor the state of the absurdist object and updates various plots every once in a while.
    class AnimationThread extends Thread 
    {
	// flag to be set after the computation thread completes, and the animation thread is no longer needed
	public boolean finished = false;

	// Run the animation.
	public void run()
	{
	    debugln( "Animation thread: started running" );

	    while( !finished ) {
		if ( absurdist != null && absurdist.isIterating() ) {		  
		    // update graph panel and plot panel during animation
		    plotPanel.setPlotSteps( absurdist.nStep );	    
		    		    
		    // Ensure that painting is triggered within 100 msec
		    graphPanel.repaint( 0, 0, graphPanel.getWidth(), graphPanel.getHeight() );
		    plotPanel.repaint( 0, 0, plotPanel.getWidth(), plotPanel.getHeight() );
		    // ??? repaint() should be fine

		    /*
		    final int maxDelay = 100;
		    graphPanel.repaint(maxDelay, 0,0, graphPanel.getWidth(),graphPanel.getHeight());
		    plotPanel.repaint(maxDelay, 0,0, plotPanel.getWidth(),plotPanel.getHeight());
		    */		    
		}
				
		/*
		  // We don't need to do this within the animation thread,
		  // because this needs to do be done only once, and it
		  // is already done from actionPerfomed() after Absurdist mapping
		  // terminates

		  if ( absurdist != null && absurdist.isMapped() ) {
		  // update graph panel and plot panel to highlight mapping result
		  plotPanel.setPlotSteps( absurdist.nStep );
		  graphPanel.repaint();
		  plotPanel.repaint();
		}	
		*/

		// try to sleep for a bit
		try { 
		    Thread.sleep( DT ); 
		}
		catch ( Exception ex ) {
		    //// System.out.println( "Run time exception in running the animation" );
		    //// ex.printStackTrace();
		    return; // this terminates the thread
		}		
	    }

	    debugln( "Animation thread: finished running" );	    
	}     
    }
    
    // Thread to run the mapping algorithms.
    class ComputationThread extends Thread 
    {
	AnimationThread animThread;
	public ComputationThread( AnimationThread animThread ) 
	{
	    this.animThread = animThread;
	}

	// Runs the computation
	public void run() 
	{
	    debugln( "Computation thread: started running" );

	    try {
		int algMode = algCombo.getSelectedIndex();	
		if ( algMode == ABS1 || algMode == EXP ) {
		    absurdist.mapSystem();
		    absurdist.printCorrespond();
		    absurdist.printMap();
		} 
		else {
		    double K1 = 100;
		    double kappa = Math.max( K1*10, 100 );
		    debug( "Use Penalty Minimizer, K1 = " + K1 + ", kappa = " + kappa );
		    penalty.PenaltyMinimizer pm = new PenaltyMinimizer();
		    pm.K1 = K1;
		    pm.kappa = kappa;
		    penalty.PenaltyMinimizer matcher = pm;
		    matcher.mapSystem( absurdist.systemO, absurdist.systemN );
		    matcher.print(false); // no file output
		}	    	    		
		// ??? don't need repaint here
		graphPanel.repaint();
		plotPanel.repaint(); 
	    } 
	    catch ( Exception ex ) {
		debugln( "Exception caught: " + ex.getMessage());
		ex.printStackTrace();
		return;
	    } 
	    finally {
		// tell the animation thread to finish
		animThread.finished = true;
		// bring buttons back to life
		mapButton.setEnabled( true );
		copyButton.setEnabled( true );
		genButton.setEnabled( true );
		debugln( "Computation thread: finished running" );
	    }
	}
    }

    /** Initializes Absurdist with two ConceptSystems loaded from a file */
    void loadSystems(String inFile) {
	try {

	absurdist = Absurdist.loadSystems(inFile, graphPanel.getWidth(), 
					  graphPanel.getHeight() );

	if (absurdist != null) {
	    graphPanel.setAbsurdist( absurdist );
	    plotPanel.setAbsurdist( absurdist );
            graphPanel.repaint();
            plotPanel.repaint();
	}
	}
	catch ( Exception ex ) {
	    ex.printStackTrace();
	    System.exit(-1);
	}	    
    }

    /** Import the source concept system from a text file. */
    void importSourceSystem( String filename )
    {
	// noise parameters
	double noiseCC = (double)ccnSlider.getValue() / 1000; // [ 0, 10% ]
	double noiseCI = (double)cinSlider.getValue() / 1000; // [ 0, 10% ]
	double noiseRC = (double)rcnSlider.getValue() / 1000; // [ 0, 10% ]
	double noiseRI = (double)rinSlider.getValue() / 1000; // [ 0, 10% ]
	
	ConceptSystem systemO;
	try {
	    systemO = ConceptSystem.readTxt( filename );
	}
	catch ( Exception ex ) {
	    Alert.alert(ex.getMessage());
	    ex.printStackTrace();		
	    return;
	}

	try {
	    ConceptSystem systemN = new ConceptSystem( systemO, noiseCC, noiseCI, noiseRC, noiseRI );
	    initAbsurdist(systemO, systemN);
	}
	catch ( Exception ex ) {
	    ex.printStackTrace();		
	    System.exit(-1);
	}
    }    

    /** Imports the target concept system from a text file. */
    void importTargetSystem( String filename ) {
	if (absurdist==null) 
	    throw new IllegalArgumentException("Absurdist must be created prior to importing 2nd system!");

	ConceptSystem systemO = absurdist.systemO;
	ConceptSystem systemN;

	try {
	    systemN = ConceptSystem.readTxt( filename );
	}
	catch ( Exception ex ) {
	    Alert.alert(ex.getMessage());
	    ex.printStackTrace();		
	    return;
	}

	try {
	    initAbsurdist(systemO, systemN);
	}
	catch ( Exception ex ) {
	    ex.printStackTrace();
	    System.exit(-1);
	}	
    }

    /** Initializes Absurdist based on 2 systems */
    private void initAbsurdist( ConceptSystem systemO, ConceptSystem systemN) 
    {
	try {    
	    ConceptGraph graphO = 
		(systemO instanceof ConceptGraph) ? (ConceptGraph) systemO:
		new ConceptGraph( systemO, 0, 0, graphPanel.getWidth(), 
				  graphPanel.getHeight()/2 );
	    ConceptGraph graphN = 
		(systemN instanceof ConceptGraph) ? (ConceptGraph) systemN:
		new ConceptGraph( systemN, 0, graphPanel.getHeight()/2, 
				  graphPanel.getWidth(), graphPanel.getHeight()/2);
	    absurdist = new Absurdist( graphO, graphN );
	    absurdist.printSystems();
	    graphPanel.setAbsurdist( absurdist );
	    plotPanel.setAbsurdist( absurdist );
            graphPanel.repaint();
            plotPanel.repaint();
	}
	catch ( Exception ex ) {
	    ex.printStackTrace();
	    System.exit(-1);
	}	
    }
}
