///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// ABSURDIST++
// Ying Feng
// Computer Scicence, Indiana University
// Feb-Aug 2003
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


 
package gui;

import java.awt.*;
//// import java.awt.geom.*;

import absurdist.*;
import driver.*;
import java.text.*;

/** Class Plot:
   Visual representation of a plot for an Absurdist variable.
*/
public class Plot
{
    /*
    static final int STEP = 500;        // maximum step on x-axis
    static final double MAXV = 1.0;     // maximum variable value on y-axis
    static final int NS = 10;           // number of scales on axises 
    */

    //static final int OX = 20, OY = 20;  // offest of coordinate origin
    static final int OX = 40, OY1 = 20, OY2=20;  // offest of coordinate origin 
 
    /** Number of pixels each step (on x-axis) or value unit (on y-axis) occupies
     */
    final int DX = 1;
    //final int DX = 20; // change scale for testing. 
    double DY = 100;  // will recompute to fit the graph
    /** Number of pixels each of x/y axis scale occupies */
    final int SX = 50;
    //int SY = 20;  // will recompute

    double[][][] var;            // variable the plot is for
    int nStep = 0;               // the actual # of steps recorded in var
    
    int x = 0, y = 0;            // top-left corner of its bound
    int width = 0, height = 0;   // width and height of its bound
    //// Rectangle bound;        // defines the bounding borders of the plot 
    
    /** Creates a plot from a variable array. */
    public Plot( double[][][] var, int nStep, int x, int y, int width, int height )
    {
	this.var = var;
	this.nStep = nStep;
	this.x = x;
	this.y = y;
	this.width = width;
	this.height = height;
	//// bound = new Rectangle( x, y, width, height );
    }

    /** Sets the variable array. */
    public void setVar( double[][][] var )
    {
        this.var = var;
    }
    
    /** Sets the number of steps. */
    public void setSteps( int nStep )
    {
        this.nStep = nStep;
    }


    /** Returns a pair numbers that encloses the entire range
	of var[][][]
    */
    int[] getRange() {
	double minVar=0, maxVar=0;

	int no = var[0].length;
	int nn = var[0][0].length;

	if (nStep < 1) maxVar=1;
	else {
	    for ( int step = 1; step < nStep; step++ ) {
		for ( int io=0; io<no; io++ ) {
		    for ( int in=0; in<nn; in++ ) {
			double v = var[step-1][io][in];
			if (v < minVar) minVar=v;
			if (v > maxVar) maxVar=v;		
		    }
		}
	    }	
	}
	maxVar= Math.ceil(maxVar); // round up 
	minVar= Math.floor(minVar);

	int [] bounds = { (int)minVar, (int)maxVar};
	return bounds;
    }


        
    /** Paints the object.
	The formula for mapping v to y:
	<pre>
	y = oy - DY * (v - minVar);
	</pre>
	where 
	<pre>
	DY = H/(maxVar-minVar);
	H=(height - OY) 
	</pre>
    */
    public void paint( Graphics2D g )
    {
	// frame
        g.setColor( Color.lightGray );
	g.setStroke( new BasicStroke(5) );
	g.drawRect( x, y, width, height );
        //// g.drawRect( bound.getX(), bound.getY(), bound.getWidth(), bound.getHeight() );

	int ox = x + OX;            // x-axis goes right from origin
	int H = height-OY1-OY2;
	int top = y + OY1;                // top end of y-axis
	int oy = top + H;   // y-axis goes down from origin
	int right = x + width;      // right end of x-axis
	int dStepScale = SX / DX;             // interval of step scale

	// Adjustable scale (V.M., Jul-2-2003)
	final int varRange[] =getRange();
	final int minVar=varRange[0];
	// to avoid zero divide
	final int maxVar=(varRange[1] > 0 ? varRange[1] : 1);

	final int range = maxVar - minVar;
	
	DY = H/range;

	Font oldFont = g.getFont();
	Font font = oldFont.deriveFont(12);
	g.setFont( font );

	// x-axis and scales (iteration steps)	
        g.setColor( Color.black );	
        g.drawLine( ox, oy, right, oy );
	int sx = ox;       // current scale xPos
	int stepScale = 0; // current step scale
	while ( sx < right ) {
	    String stepStr = ( new Integer( stepScale ) ).toString(); 
	    g.drawString( stepStr, sx, y-2 );
	    sx += SX;
	    stepScale += dStepScale * Absurdist.intLoop;
	}	    
	/*
	for ( int i=0; i<NS-1; i++ ) {
	    String stepStr = ( new Integer( i * dStep ) ).toString(); 
	    int scaleX = i * dx;
	    g.drawString( stepStr, scaleX, oy );
	}
	*/

	// y-axis and scales (variable values)	
        g.setColor( Color.black );	
        g.drawLine( ox, oy, ox, top );

	// Formatting numbers
	DecimalFormat df = new DecimalFormat("0.00");

	int NSV = 4;
	int jmin = (minVar * NSV) / maxVar;
		
	for(int j=jmin ; j<=NSV; j++) {
	    double v = (j * maxVar)/(double)NSV;
	    int sy  = oy - (int)((v - minVar)*DY);
	    String valStr = df.format( v );
	    g.drawString( valStr, 0, sy );
	}

	g.setFont( oldFont );
	
	// line plots
	g.setStroke( new BasicStroke(1) );
	int no = var[0].length;
	int nn = var[0][0].length;
	for ( int step = 1; step < nStep; step++ ) {
	    int xs = ox + ( step - 1 ) * DX;
	    int xe = ox + step * DX;
	    for ( int io=0; io<no; io++ ) 
		for ( int in=0; in<nn; in++ ) {
		    int ys = oy - ( int )( (var[step-1][io][in]-minVar) * DY );
		    int ye = oy - ( int )( (var[step][io][in]-minVar) * DY );
		    g.drawLine( xs, ys, xe, ye );
		}
	}
    }
}
